from _incydr_sdk.enums.file_events import Category
from _incydr_sdk.enums.file_events import EventAction
from _incydr_sdk.enums.file_events import EventSearchTerm
from _incydr_sdk.enums.file_events import FileCategory
from _incydr_sdk.enums.file_events import ReportType
from _incydr_sdk.enums.file_events import RiskIndicators
from _incydr_sdk.enums.file_events import RiskSeverity
from _incydr_sdk.enums.file_events import ShareType
from _incydr_sdk.enums.file_events import TrustReason

__all__ = [
    "EventSearchTerm",
    "FileCategory",
    "EventAction",
    "Category",
    "ShareType",
    "ReportType",
    "RiskIndicators",
    "TrustReason",
    "RiskSeverity",
]



import chardet
import click


class AutoDecodedFile(click.File):
    """Attempts to autodetect file's encoding prior to normal click.File processing."""

    def convert(self, value, param, ctx):
        try:
            with open(value, "rb") as file:
                self.encoding = chardet.detect(file.read())["encoding"]
        except Exception:
            pass  # we'll let click.File do its own exception handling for the filepath

        return super().convert(value, param, ctx)


class FileOrString(AutoDecodedFile):
    """Declares a parameter to be a file (if the argument begins with `@`), otherwise accepts it as
    a string.
    """

    def __init__(self):
        super().__init__("r")

    def convert(self, value, param, ctx):
        if value.startswith("@") or value == "-":
            value = value.lstrip("@")
            return super().convert(value, param, ctx)
        else:
            return value



import click
from rich.panel import Panel

from _incydr_cli import console
from _incydr_cli import logging_options
from _incydr_cli import render
from _incydr_cli.cmds.options.output_options import columns_option
from _incydr_cli.cmds.options.output_options import single_format_option
from _incydr_cli.cmds.options.output_options import SingleFormat
from _incydr_cli.cmds.options.output_options import table_format_option
from _incydr_cli.cmds.options.output_options import TableFormat
from _incydr_cli.core import incompatible_with
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_sdk.actors.client import ActorNotFoundError
from _incydr_sdk.actors.models import Actor
from _incydr_sdk.core.client import Client
from _incydr_sdk.exceptions import DateParseError
from _incydr_sdk.utils import model_as_card


@click.group(cls=IncydrGroup)
@logging_options
def actors():
    """
    View and manage Incydr actors.

    Actors are identities that generate events collected by Incydr.
    User cloud accounts are consolidated into a single actor through adoption,
    which tie related actors to one parent actor.

    The actor commands can be used to retrieve information about actors and their relationships.
    """


@actors.command("list", cls=IncydrCommand)
@click.option(
    "--active/--inactive",
    default=None,
    help="Filter by active or inactive actors. Defaults to returning both when when neither option is passed.",
)
@click.option(
    "--name-starts-with",
    default=None,
    help="Find actors whose name (e.g. username/email) starts with this text, ignoring case.",
)
@click.option(
    "--name-ends-with",
    default=None,
    help="Find actors whose name (e.g. username/email) ends with this text, ignoring case.",
)
@click.option(
    "--prefer-parent",
    is_flag=True,
    default=False,
    help="Returns an actor's parent when applicable. Returns an actor themselves if they have no parent.",
)
@table_format_option
@columns_option
@logging_options
def list_(
    active: bool = None,
    name_starts_with: str = None,
    name_ends_with: str = None,
    prefer_parent: bool = None,
    format_: TableFormat = None,
    columns: str = None,
):
    """
    List actors.
    """

    client = Client()

    actors = client.actors.v1.iter_all(
        active=active,
        name_starts_with=name_starts_with,
        name_ends_with=name_ends_with,
        prefer_parent=prefer_parent,
    )

    if format_ == TableFormat.table:
        render.table(Actor, actors, columns=columns, flat=False)
    elif format_ == TableFormat.csv:
        render.csv(Actor, actors, columns=columns, flat=True)
    elif format_ == TableFormat.json_pretty:
        for actor in actors:
            console.print_json(actor.json())
    else:
        for actor in actors:
            click.echo(actor.json())


@actors.command(cls=IncydrCommand)
@click.option("--actor-id", default=None, help="Get an actor by actor ID.")
@click.option(
    "--name",
    default=None,
    help="Get an actor by actor name, typically a username/email address.",
)
@click.option(
    "--prefer-parent",
    is_flag=True,
    default=False,
    help="Returns an actor's parent when applicable. Returns an actor themselves if they have no parent.",
)
@single_format_option
@logging_options
def show(
    actor_id: str = None,
    name: str = None,
    prefer_parent: bool = None,
    format_: SingleFormat = None,
):
    """
    Show actor details.

    Specify actor by ID or name, either --actor-id or --name is required.
    """
    client = Client()

    try:
        if actor_id:
            if name:
                console.print(
                    "Both --actor-id and --name options passed, ignoring name and retrieving by actor ID."
                )
            actor = client.actors.v1.get_actor_by_id(
                actor_id, prefer_parent=prefer_parent
            )
        elif name:
            actor = client.actors.v1.get_actor_by_name(
                name, prefer_parent=prefer_parent
            )
        else:
            raise click.UsageError(
                "At least one of --actor-id or --name options must be supplied."
            )
    except ActorNotFoundError:
        console.print(f"No results for actors with name '{name}'.")
        return

    if format_ == SingleFormat.rich and client.settings.use_rich:
        console.print(Panel.fit(model_as_card(actor), title=f"Actor {actor.actor_id}"))
    elif format_ == SingleFormat.json_pretty:
        console.print_json(actor.json())
    else:
        click.echo(actor.json())


@actors.command(cls=IncydrCommand)
@click.option(
    "--actor-id",
    default=None,
    help="Get an actor family by the actor ID of one of its members.",
)
@click.option(
    "--name",
    default=None,
    help="Get an actor family by the actor name of one of its members, where name is typically a username/email address.",
)
@single_format_option
@logging_options
def show_family(actor_id: str = None, name: str = None, format_: SingleFormat = None):
    """
    Show an actors family.

    Specify actor by ID or name, either --actor-id or --name is required.

    An actor family consists of the the parent actor and any associated children.
    """

    client = Client()
    if actor_id:
        if name:
            console.print(
                "Both --actor-id and --name options passed, ignoring name and retrieving by actor ID."
            )
        family = client.actors.v1.get_family_by_member_id(actor_id)
    elif name:
        family = client.actors.v1.get_family_by_member_name(name)
    else:
        raise click.UsageError(
            "At least one of --actor-id or --name options must be supplied."
        )

    if format_ == SingleFormat.rich and client.settings.use_rich:
        console.print(
            Panel.fit(
                model_as_card(family),
                title=f"Actor Family with Parent {family.parent.actor_id}",
            )
        )
    elif format_ == SingleFormat.json_pretty:
        console.print_json(family.json())
    else:
        click.echo(family.json())


@actors.command(cls=IncydrCommand)
@click.argument("actor-id")
@click.option(
    "--start-date",
    default=None,
    help="Update a user's starting date. Accepts a date in yyyy-MM-dd (UTC) format.",
)
@click.option(
    "--end-date",
    default=None,
    help="Update a user's departure date. Accepts a date in yyyy-MM-dd (UTC) format.",
)
@click.option(
    "--notes", default=None, help="Update the optional notes on a user's profile."
)
@click.option(
    "--clear-start-date",
    is_flag=True,
    default=False,
    help="Clear the start date on a user's profile. Incompatible with --start-date.",
    cls=incompatible_with("start_date"),
)
@click.option(
    "--clear-end-date",
    is_flag=True,
    default=False,
    help="Clear the end date on a user's profile. Incompatible with --end-date.",
    cls=incompatible_with("end_date"),
)
@click.option(
    "--clear-notes",
    is_flag=True,
    default=False,
    help="Clear the notes on a user's profile. Incompatible with --notes.",
    cls=incompatible_with("notes"),
)
@logging_options
def update(
    actor_id,
    start_date=None,
    end_date=None,
    notes=None,
    clear_start_date=None,
    clear_end_date=None,
    clear_notes=None,
):
    """
    Update an actor.

    Accepts actor ID.
    """
    if not any(
        [start_date, end_date, notes, clear_start_date, clear_end_date, clear_notes]
    ):
        raise click.UsageError(
            "At least one of --start-date, --end-date, or --notes, or one of their corresponding clear flags, "
            "is required to update an actor."
        )

    client = Client()

    if clear_start_date:
        start_date = ""
    if clear_end_date:
        end_date = ""
    if clear_notes:
        notes = ""

    try:
        updated_profile = client.actors.v1.update_actor(
            actor=actor_id, notes=notes, start_date=start_date, end_date=end_date
        )
        if client.settings.use_rich:
            console.print(
                Panel.fit(model_as_card(updated_profile), title="Updated Actor")
            )
        else:
            console.print(updated_profile.json(), highlight=False)
    except DateParseError as err:
        raise err



from datetime import datetime
from itertools import count
from pathlib import Path
from typing import List
from typing import Union

from _incydr_sdk.audit_log.models import AuditEventsPage
from _incydr_sdk.audit_log.models import DateRange
from _incydr_sdk.audit_log.models import QueryAuditLogRequest
from _incydr_sdk.audit_log.models import QueryExportRequest
from _incydr_sdk.audit_log.models import UserTypes
from _incydr_sdk.core.utils import get_filename_from_content_disposition
from _incydr_sdk.queries.utils import parse_ts_to_posix_ts


class AuditLogClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = AuditLogV1(self._parent)
        return self._v1


class AuditLogV1:
    """
    Client for `/v1/audit` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.audit_log.v1.get_page()
    """

    def __init__(self, parent):
        self._parent = parent

    def get_page(
        self,
        page_num: int = 0,
        page_size: int = None,
        actor_ids: Union[List[str], str] = None,
        actor_ip_addresses: Union[List[str], str] = None,
        actor_names: Union[List[str], str] = None,
        start_time: Union[str, datetime] = None,
        end_time: Union[str, datetime] = None,
        event_types: Union[List[str], str] = None,
        resource_ids: Union[List[str], str] = None,
        user_types: Union[List[UserTypes], UserTypes] = None,
    ) -> AuditEventsPage:
        """
        Search audit log entries.

        **Parameters:**

        * **page_num**: `int` - page_num number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return per page. Defaults to client's `page_size` setting.
            Maximum page size is 10,000.
        * **actor_ids**: `List[str] | str` - Finds events whose actor_id is one of the given ids.
        * **actor_ip_addresses**: `List[str] | str` - Finds events whose actor_ip_address is one of the given IP addresses.
        * **actor_names**: `List[str] | str` - Finds events whose actor_name is one of the given names.
        * **start_time**: `datetime | str` - Search for events within a date range.  Start time for this date range.
        * **end_time**: `datetime | str` - Search for events within a date range.  End time for this date range.
        * **event_types**: `List[str] | str` - Finds events whose type is one of the given types.
        * **resource_ids**: `List[str] | str` - Filters searchable events that match resource_id.
        * **user_types**: `List[UserTypes]` - Filters searchable events that match actor type.

        **Returns**: A [`AuditEventsPage`][auditeventspage-model] object representing the search response.
        """

        page_size = page_size or self._parent.settings.page_size

        request = _build_query_request(
            page_num=page_num,
            page_size=page_size,
            actor_ids=actor_ids,
            actor_ip_addresses=actor_ip_addresses,
            actor_names=actor_names,
            start_time=start_time,
            end_time=end_time,
            event_types=event_types,
            resource_ids=resource_ids,
            user_types=user_types,
        )

        response = self._parent.session.post(
            "/v1/audit/search-audit-log", json=request.dict()
        )

        return AuditEventsPage.parse_response(response)

    def iter_all(
        self,
        page_size: int = None,
        actor_ids: Union[List[str], str] = None,
        actor_ip_addresses: Union[List[str], str] = None,
        actor_names: Union[List[str], str] = None,
        start_time: Union[str, datetime] = None,
        end_time: Union[str, datetime] = None,
        event_types: Union[List[str], str] = None,
        resource_ids: Union[List[str], str] = None,
        user_types: Union[List[UserTypes], UserTypes] = None,
    ):
        """
        Iterate over all audit log events.

        Accepts the same parameters as `.get_page()` except `page_num`.

        **Returns**: A generator yielding individual `dict` objects representing audit log events.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(0):
            page = self.get_page(
                page_num=page_num,
                page_size=page_size,
                actor_ids=actor_ids,
                actor_ip_addresses=actor_ip_addresses,
                actor_names=actor_names,
                start_time=start_time,
                end_time=end_time,
                event_types=event_types,
                resource_ids=resource_ids,
                user_types=user_types,
            )
            yield from page.events
            if len(page.events) < page_size:
                break

    def get_event_count(
        self,
        page_num: int = 0,
        page_size: int = None,
        actor_ids: Union[List[str], str] = None,
        actor_ip_addresses: Union[List[str], str] = None,
        actor_names: Union[List[str], str] = None,
        start_time: Union[str, datetime] = None,
        end_time: Union[str, datetime] = None,
        event_types: Union[List[str], str] = None,
        resource_ids: Union[List[str], str] = None,
        user_types: Union[List[UserTypes], UserTypes] = None,
    ) -> int:
        """
        Get the total result count of a search.

        **Parameters:**

        * **page_num**: `int` - Page number for results, starting at 0.
        * **page_size**: `int` - Max number of results to return per page.
        * **actor_ids**: `List[str] | str` - Finds events whose actor_id is one of the given ids.
        * **actor_ip_addresses**: `List[str] | str` - Finds events whose actor_ip_address is one of the given IP addresses.
        * **actor_names**: `List[str] | str` - Finds events whose actor_name is one of the given names.
        * **start_time**: `datetime | str` - Search for events within a date range.  Start time for this date range.
        * **end_time**: `datetime | str` - Search for events within a date range.  End time for this date range.
        * **event_types**: `List[str] | str` - Finds events whose type is one of the given types.
        * **resource_ids**: `List[str] | str` - Filters searchable events that match resource_id.
        * **user_types**: `List[UserTypes]` - Filters searchable events that match actor type.

        **Returns**: An `int` indicating the number of resulting audit log events from search.
        """

        page_size = page_size or self._parent.settings.page_size

        request = _build_query_request(
            page_num=page_num,
            page_size=page_size,
            actor_ids=actor_ids,
            actor_ip_addresses=actor_ip_addresses,
            actor_names=actor_names,
            start_time=start_time,
            end_time=end_time,
            event_types=event_types,
            resource_ids=resource_ids,
            user_types=user_types,
        )

        response = self._parent.session.post(
            "/v1/audit/search-results-count", json=request.dict()
        )

        return response.json()["totalResultCount"]

    def download_events(
        self,
        target_folder: Path,
        actor_ids: Union[List[str], str] = None,
        actor_ip_addresses: Union[List[str], str] = None,
        actor_names: Union[List[str], str] = None,
        start_time: Union[str, datetime] = None,
        end_time: Union[str, datetime] = None,
        event_types: Union[List[str], str] = None,
        resource_ids: Union[List[str], str] = None,
        user_types: Union[List[UserTypes], UserTypes] = None,
    ) -> Path:
        """
        Export search results.

        **Parameters:**

        * **target_folder**: `Path, str` (required) - A string or `pathlib.Path` object that represents the folder
        which the file will be saved to.
        * **actor_ids**: `List[str] | str` - Finds events whose actor_id is one of the given ids.
        * **actor_ip_addresses**: `List[str] | str` - Finds events whose actor_ip_address is one of the given IP addresses.
        * **actor_names**: `List[str] | str` - Finds events whose actor_name is one of the given names.
        * **start_time**: `datetime | str` - Search for events within a date range.  Start time for this date range.
        * **end_time**: `datetime | str` - Search for events within a date range.  End time for this date range.
        * **event_types**: `List[str] | str` - Finds events whose type is one of the given types.
        * **resource_ids**: `List[str] | str` - Filters searchable events that match resource_id.
        * **user_types**: `List[UserTypes]` - Filters searchable events that match actor type.

        **Returns**: A `pathlib.Path` object representing location of the downloaded csv file.
        """

        date_range = DateRange()
        if start_time:
            date_range.startTime = parse_ts_to_posix_ts(start_time)
        if end_time:
            date_range.endTime = parse_ts_to_posix_ts(end_time)

        data = QueryExportRequest(
            actorIds=[actor_ids] if isinstance(actor_ids, str) else actor_ids,
            actorIpAddresses=[actor_ip_addresses]
            if isinstance(actor_ip_addresses, str)
            else actor_ip_addresses,
            actorNames=[actor_names] if isinstance(actor_names, str) else actor_names,
            dateRange=date_range,
            eventTypes=[event_types] if isinstance(event_types, str) else event_types,
            resourceIds=[resource_ids]
            if isinstance(resource_ids, str)
            else resource_ids,
            userTypes=[user_types] if isinstance(user_types, str) else user_types,
        )

        folder = Path(target_folder)  # ensure a Path object if we get passed a string
        if not folder.is_dir():
            raise ValueError(
                f"`target_folder` argument must resolve to a folder: {target_folder}"
            )
        export_response = self._parent.session.post(
            "/v1/audit/export", json=data.dict()
        )

        download_response = self._parent.session.get(
            f"/v1/audit/redeem-download-token?downloadToken={export_response.json()['downloadToken']}"
        )

        filename = get_filename_from_content_disposition(
            download_response, fallback="AuditLog_SearchResults.csv"
        )
        target = folder / filename
        target.write_bytes(download_response.content)

        return target


def _build_query_request(
    page_num: int = 0,
    page_size: int = 100,
    actor_ids: Union[List[str], str] = None,
    actor_ip_addresses: Union[List[str], str] = None,
    actor_names: Union[List[str], str] = None,
    start_time: Union[str, datetime] = None,
    end_time: Union[str, datetime] = None,
    event_types: Union[List[str], str] = None,
    resource_ids: Union[List[str], str] = None,
    user_types: Union[List[UserTypes], UserTypes] = None,
):
    date_range = DateRange()
    if start_time:
        date_range.startTime = parse_ts_to_posix_ts(start_time)
    if end_time:
        date_range.endTime = parse_ts_to_posix_ts(end_time)

    request = QueryAuditLogRequest(
        actorIds=[actor_ids] if isinstance(actor_ids, str) else actor_ids,
        actorIpAddresses=[actor_ip_addresses]
        if isinstance(actor_ip_addresses, str)
        else actor_ip_addresses,
        actorNames=[actor_names] if isinstance(actor_names, str) else actor_names,
        dateRange=date_range,
        eventTypes=[event_types] if isinstance(event_types, str) else event_types,
        page=page_num,
        pageSize=page_size,
        resourceIds=[resource_ids] if isinstance(resource_ids, str) else resource_ids,
        userTypes=[user_types] if isinstance(user_types, str) else user_types,
    )

    return request



from contextlib import nullcontext
from datetime import timezone
from typing import Optional
from typing import Union

import click
import requests
from boltons.iterutils import bucketize
from boltons.iterutils import chunked
from click import BadOptionUsage
from click import File
from pydantic import Field
from rich.panel import Panel

from _incydr_cli import console
from _incydr_cli import get_user_project_path
from _incydr_cli import logging_options
from _incydr_cli import render
from _incydr_cli.cmds.options.alert_filter_options import advanced_query_option
from _incydr_cli.cmds.options.alert_filter_options import filter_options
from _incydr_cli.cmds.options.output_options import columns_option
from _incydr_cli.cmds.options.output_options import input_format_option
from _incydr_cli.cmds.options.output_options import output_options
from _incydr_cli.cmds.options.output_options import single_format_option
from _incydr_cli.cmds.options.output_options import SingleFormat
from _incydr_cli.cmds.options.output_options import table_format_option
from _incydr_cli.cmds.options.output_options import TableFormat
from _incydr_cli.cmds.options.utils import checkpoint_option
from _incydr_cli.cmds.utils import deprecation_warning
from _incydr_cli.cmds.utils import warn_interrupt
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_cli.cursor import CursorStore
from _incydr_cli.file_readers import AutoDecodedFile
from _incydr_cli.logger import get_server_logger
from _incydr_sdk.alerts.models.alert import AlertSummary
from _incydr_sdk.core.client import Client
from _incydr_sdk.core.models import CSVModel
from _incydr_sdk.core.models import Model
from _incydr_sdk.queries.alerts import AlertQuery
from _incydr_sdk.utils import model_as_card

# Deprecated April 2024.
DEPRECATION_TEXT = "DeprecationWarning: Alerts commands are deprecated. Use the 'incydr sessions' command group instead."


@click.group(cls=IncydrGroup)
@logging_options
def alerts():
    """DEPRECATED. Use the Sessions command group instead. View and manage alerts."""
    deprecation_warning(DEPRECATION_TEXT)


@alerts.command(cls=IncydrCommand)
@checkpoint_option
@logging_options
@columns_option
@table_format_option
@output_options
@advanced_query_option
@filter_options
def search(
    format_: TableFormat,
    columns: Optional[str],
    output: Optional[str],
    certs: Optional[str],
    ignore_cert_validation: Optional[bool],
    advanced_query: Optional[Union[str, File]],
    start: Optional[str],
    end: Optional[str],
    on: Optional[str],
    alert_id: Optional[str],
    type_: Optional[str],
    name: Optional[str],
    actor: Optional[str],
    actor_id: Optional[str],
    risk_severity: Optional[str],
    state: Optional[str],
    rule_id: Optional[str],
    alert_severity: Optional[str],
    checkpoint_name: Optional[str],
):
    """
    Search alerts.  Various options are provided to filter query results.

    Results will be output to the console by default, use the `--output` option to send data to a server.

    Checkpointing is available through the `--checkpoint <checkpoint-name>` option and will only return new results
    on subsequent queries with that same checkpoint.  Checkpointing filters by timestamp, additional filter
    options will need to be included in each run.
    """
    client = Client()
    cursor = _get_cursor_store(client.settings.api_client_id)

    if output:
        format_ = TableFormat.json_lines

    # Use stored checkpoint timestamp for start filter if applicable
    if checkpoint_name:
        checkpoint = cursor.get(checkpoint_name)
        if checkpoint:
            start = float(checkpoint)

    if advanced_query:
        if not isinstance(advanced_query, str):
            advanced_query = advanced_query.read()
        query = AlertQuery.parse_raw(advanced_query)
    else:
        if not any([start, on, end]):
            raise BadOptionUsage(
                "start",
                "--start, --end, or --on options are required if not using the --advanced-query option "
                "or using an existing checkpoint.",
            )
        query = _create_query(
            start=start,
            end=end,
            on=on,
            alert_id=alert_id,
            type_=type_,
            name=name,
            actor=actor,
            actor_id=actor_id,
            risk_severity=risk_severity,
            state=state,
            rule_id=rule_id,
            alert_severity=alert_severity,
        )
    alerts_gen = client.alerts.v1.iter_all(query)

    if checkpoint_name:
        alerts_gen = _update_checkpoint(cursor, checkpoint_name, alerts_gen)

    with warn_interrupt() if checkpoint_name else nullcontext():
        if output:
            logger = get_server_logger(output, certs, ignore_cert_validation)
            for alert_ in alerts_gen:
                logger.info(alert_.json())
            return

        if format_ == TableFormat.table:
            columns = columns or [
                "created_at",
                "risk_severity",
                "state",
                "actor",
                "actor_id",
                "name",
                "description",
                "watchlists",
                "state_last_modified_by",
                "state_last_modified_at",
                "id",
            ]
            render.table(AlertSummary, alerts_gen, columns=columns, flat=False)
        elif format_ == TableFormat.csv:
            render.csv(AlertSummary, alerts_gen, columns=columns, flat=True)
        else:
            printed = False
            for alert_ in alerts_gen:
                printed = True
                if format_ == TableFormat.json_pretty:
                    console.print_json(alert_.json())
                else:
                    click.echo(alert_.json())
            if not printed:
                console.print("No results found.")


@alerts.command()
@click.argument("checkpoint-name")
def clear_checkpoint(checkpoint_name: str):
    """Remove the saved alerts checkpoint from searches made with `--checkpoint` mode."""
    client = Client()
    cursor = _get_cursor_store(client.settings.api_client_id)
    cursor.delete(checkpoint_name)


# Future enhancement: add functionality to show human-readable summaries for multiple alerts
@alerts.command(cls=IncydrCommand)
@logging_options
@single_format_option
@click.argument("alert-id")
def show(
    alert_id: str,
    format_: SingleFormat,
):
    """
    Show the details of a single alert.
    """
    client = Client()
    alert = client.alerts.v1.get_details(alert_id)[0]
    if format_ == SingleFormat.rich:
        console.print(Panel.fit(model_as_card(alert)))
    elif format_ == SingleFormat.json_pretty:
        console.print_json(alert.json())
    else:
        click.echo(alert.json())


@alerts.command(cls=IncydrCommand)
@logging_options
@click.argument("alert-id")
@click.argument("note")
def add_note(
    alert_id: str,
    note: str,
):
    """
    Add an optional note to an alert.
    """
    client = Client()
    client.alerts.v1.add_note(alert_id, note)
    console.print("Note added.")


@alerts.command(cls=IncydrCommand)
@logging_options
@click.argument("alert-id")
@click.argument("state")
@click.option(
    "--note",
    default=None,
    help="Optional note to indicate the reason for the state change.",
)
def update_state(
    alert_id: str,
    state: str,
    note: Optional[str],
):
    """
    Change the state of an alert, and optionally add a note.
    """
    client = Client()
    client.alerts.v1.change_state(alert_id, state, note)
    console.print("State changed successfully.")


@alerts.command(cls=IncydrCommand)
@click.argument("file", type=AutoDecodedFile())
@input_format_option
@click.option(
    "--state",
    type=click.Choice(["OPEN", "RESOLVED", "IN_PROGRESS", "PENDING"]),
    help="Override CSV/JSON input's `state` value with this value.",
)
@click.option("--note", help="Override CSV/JSON input's `note` value with this value.")
@logging_options
def bulk_update_state(
    file,
    format_,
    state: str,
    note: str,
):
    """
    Bulk update multiple alerts from a file.

    Takes a single arg `FILE` which specifies the path to the file (use "-" to read from stdin).

    File format can either be CSV or [JSON Lines format](https://jsonlines.org) (Default is CSV).

    The --state and --note options to this command will override respective columns/keys in the CSV/JSON input.

    This allows you to bulk change a set of alerts without having manually modify the state/note value for each CSV or
    JSON Lines row in the file. For example, to close all currently "PENDING" alerts older than <DATE>:

        incydr alerts search --end <DATE> --state PENDING --format json-lines | incydr alerts bulk-update-state - --state RESOLVED --note "bulk resolved alerts older than <DATE>"

    If --state is not provided, the CSV/JSON input _must_ have a `state` column/key for each row/object.
    """
    # if --state is provided, we want that column/key to be optional on input data, otherwise required
    state_type = Optional[str] if state else str

    class AlertBulkCSV(CSVModel):
        alert_id: str = Field(csv_aliases=["id", "alert_id"])
        state: state_type = Field(None, csv_aliases=["state"])  # type: ignore
        note: Optional[str] = None

    class AlertBulkJSON(Model):
        alert_id: str = Field(alias="id")
        state: state_type = None  # type: ignore
        note: Optional[str] = None

    client = Client()
    if format_ == "csv":
        alerts_ = AlertBulkCSV.parse_csv(file)

    else:
        alerts_ = AlertBulkJSON.parse_json_lines(file)

    # group alerts where state and note are the same, so we can batch API calls
    buckets = bucketize(
        alerts_,
        key=lambda alert: (state or alert.state, note or alert.note),
        value_transform=lambda alert: alert.alert_id,
    )
    for bucket in buckets:
        state_, note_ = bucket
        alert_ids = buckets[bucket]
        # backend allows max of 100 alerts per request
        for chunk in chunked(alert_ids, size=100):
            try:
                client.alerts.v1.change_state(chunk, state_, note_)
                console.print(
                    f"{len(chunk)} alerts successfully set to '{state_}' with note: '{note_}'"
                )
            except requests.HTTPError as err:
                # backend doesn't specify _which_ alert_id doesn't exist, so we need to try them 1 by 1
                if err.response.status_code == 404:
                    console.print(
                        f"[red]Error processing batch of {len(chunk)} alerts, trying individually..."
                    )
                    for id_ in chunk:
                        try:
                            client.alerts.v1.change_state(id_, state_, note_)
                            console.print(
                                f"Successfully set alert_id '{id_}' to '{state_}' with note: '{note_}'"
                            )
                        except requests.HTTPError as err:
                            console.print(
                                f"[red]Error updating alert_id[/red] '{id_}': {err.response.status_code}"
                            )


field_option_map = {
    "alert_id": "AlertId",
    "type_": "Type",
    "name": "Name",
    "actor": "Actor",
    "actor_id": "ActorId",
    "risk_severity": "RiskSeverity",
    "state": "State",
    "rule_id": "RuleId",
    "alert_severity": "AlertSeverity",
}


def _create_query(**kwargs):
    query = AlertQuery(
        start_date=kwargs["start"], end_date=kwargs["end"], on=kwargs["on"]
    )
    for k, v in kwargs.items():
        if v:
            if k in ["start", "end", "on"]:
                continue
            query = query.equals(field_option_map[k], v)
    return query


def _get_cursor_store(api_key):
    """
    Get cursor store for alerts search checkpoints.
    """
    dir_path = get_user_project_path(
        "checkpoints",
        api_key,
        "alert_checkpoints",
    )
    return CursorStore(dir_path, "alerts")


def _update_checkpoint(cursor, checkpoint_name, alerts_gen):
    """
    De-duplicates events across checkpointed runs.

    Since using the timestamp of the last event
    processed as the `--start` time of the next run causes the last event to show up again in the
    next results, store the last alert IDs in the cursor to
    filter out on the next run.

    It's also possible that two events have the exact same timestamp, so
    `checkpoint_alerts` needs to be a list of alert IDs so we can filter out everything that's actually
    been processed.
    """
    checkpoint_alerts = cursor.get_items(checkpoint_name)
    new_timestamp = None
    new_alerts = []
    for alert in alerts_gen:
        alert_id = alert.id
        if alert_id not in checkpoint_alerts:
            if not new_timestamp or alert.created_at > new_timestamp:
                new_timestamp = alert.created_at
                new_alerts.clear()
            new_alerts.append(alert_id)
            yield alert
            new_timestamp.replace(tzinfo=timezone.utc)
            cursor.replace(checkpoint_name, new_timestamp.timestamp())
            cursor.replace_items(checkpoint_name, new_alerts)



from typing import List
from typing import Optional
from typing import Union

from pydantic import BaseModel
from pydantic import ConfigDict
from pydantic import Field
from pydantic import StringConstraints
from typing_extensions import Annotated

from _incydr_sdk.enums.watchlists import WatchlistType


class UpdateExcludedUsersRequest(BaseModel):
    userIds: Optional[List[str]] = Field(
        None,
        description="A list of user IDs to add or remove.",
        max_length=100,
    )


class UpdateExcludedActorsRequest(BaseModel):
    actorIds: Optional[List[str]] = Field(
        None,
        description="A list of actor IDs to add or remove.",
        max_length=100,
    )


class UpdateIncludedDepartmentsRequest(BaseModel):
    departments: Optional[List[str]] = Field(
        None, description="A list of departments to add or remove."
    )


class UpdateIncludedDirectoryGroupsRequest(BaseModel):
    groupIds: Optional[List[str]] = Field(
        None, description="A list of group IDs to add or remove."
    )


class UpdateIncludedUsersRequest(BaseModel):
    userIds: Optional[List[str]] = Field(
        None,
        description="A list of user IDs to add or remove.",
        max_length=100,
    )
    watchlistId: Optional[str] = Field(
        None, description="A unique watchlist ID.", examples=["123"]
    )


class UpdateIncludedActorsRequest(BaseModel):
    actorIds: Optional[List[str]] = Field(
        None,
        description="A list of actor IDs to add or remove.",
        max_length=100,
    )
    watchlistId: Optional[str] = Field(
        None, description="A unique watchlist ID.", examples=["123"]
    )


class CreateWatchlistRequest(BaseModel):
    description: Optional[Annotated[str, StringConstraints(max_length=250)]] = Field(
        None,
        description="The optional description of a custom watchlist.",
        examples=["List of users that fit a custom use case."],
    )
    title: Optional[Annotated[str, StringConstraints(max_length=50)]] = Field(
        None,
        description="The required title for a custom watchlist.",
        examples=["My Custom List"],
    )
    watchlistType: Union[WatchlistType, str]
    model_config = ConfigDict(use_enum_values=True)


class ListWatchlistsRequest(BaseModel):
    page: int = 1
    pageSize: int = 100
    userId: Optional[str] = None


class ListWatchlistsRequestV2(BaseModel):
    page: int = 1
    pageSize: int = 100
    actorId: Optional[str] = None


class UpdateWatchlistRequest(BaseModel):
    description: Optional[Annotated[str, StringConstraints(max_length=250)]] = Field(
        None,
        description="The updated description of a custom watchlist.",
        examples=["List of users that fit a custom use case."],
    )
    title: Optional[Annotated[str, StringConstraints(max_length=50)]] = Field(
        None,
        description="The updated title for a custom watchlist.",
        examples=["My Custom List"],
    )



from itertools import count
from typing import Iterator

from requests import Response

from _incydr_sdk.legal_hold.models import AddCustodianResponse
from _incydr_sdk.legal_hold.models import Custodian
from _incydr_sdk.legal_hold.models import CustodianMatter
from _incydr_sdk.legal_hold.models import CustodianMattersPage
from _incydr_sdk.legal_hold.models import CustodiansPage
from _incydr_sdk.legal_hold.models import LegalHoldPolicy
from _incydr_sdk.legal_hold.models import LegalHoldPolicyPage
from _incydr_sdk.legal_hold.models import Matter
from _incydr_sdk.legal_hold.models import MattersPage


class LegalHoldClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = LegalHoldV1(self._parent)
        return self._v1


class LegalHoldV1:
    """Client for `/v1/orgs` endpoints.

    Usage example:

        >>> import incydr
        >>>
        >>> client = incydr.Client(**kwargs)
        >>> client.legal_hold.v1.get_matter("matter_id")
    """

    def __init__(self, parent):
        self._parent = parent

    def get_memberships_page_for_user(
        self, user_id: str, page_num: int = None, page_size: int = None
    ) -> CustodianMattersPage:
        """Get a page of matter memberships for a given user

        **Parameters:**

        * **user_id**: `str` (required) - The user ID for the user being queried.
        * **page_num**: `int` - The page number to request.
        * **page_size**: `int` - The page size to request

        **Returns**: A [`CustodianMattersPage`][custodianmatterspage-model] object with the page of memberships for that user.
        """
        return CustodianMattersPage.parse_response(
            self._parent.session.get(
                f"/v1/legal-hold/custodians/{user_id}",
                params={"page": page_num, "pageSize": page_size},
            )
        )

    def iter_all_memberships_for_user(
        self, user_id: str, page_size: int = None
    ) -> Iterator[CustodianMatter]:
        """Get all matter memberships for a given user

        **Parameters:**

        * **user_id**: `str` (required) - The user ID for the user being queried.
        * **page_size**: `int` - The page size to request

        **Returns**: A generator object that yields [`CustodianMatter`][custodianmatter-model] objects with the memberships for the given user.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_memberships_page_for_user(
                user_id=user_id, page_num=page_num, page_size=page_size
            )
            yield from page.matters
            if len(page.matters) < page_size:
                break

    def get_custodians_page(
        self, matter_id: str, page_num: int = None, page_size: int = None
    ):
        """Get a page of custodians for a given matter

        **Parameters:**

        * **matter_id**: `str` (required) - The matter ID being queried.
        * **page_num**: `int` - The page number to request.
        * **page_size**: `int` - The page size to request

        **Returns**: A [`CustodiansPage`][custodianspage-model] object with the page of memberships for that matter.
        """
        return CustodiansPage.parse_response(
            self._parent.session.get(
                f"/v1/legal-hold/matters/{matter_id}/custodians",
                params={"page": page_num, "pageSize": page_size},
            )
        )

    def iter_all_custodians(
        self, matter_id: str, page_size: int = None
    ) -> Iterator[Custodian]:
        """Get all custodians for a given matter

        **Parameters:**

        * **matter_id**: `str` (required) - The matter ID being queried.
        * **page_size**: `int` - The page size to request

        **Returns**: A generator object that yields [`Custodian`][custodian-model] objects with the memberships for the given matter.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_custodians_page(
                matter_id=matter_id, page_num=page_num, page_size=page_size
            )
            yield from page.custodians
            if len(page.custodians) < page_size:
                break

    def add_custodian(self, matter_id: str, user_id: str) -> AddCustodianResponse:
        """Add a user to a matter

        **Parameters:**

        * **matter_id**: `str` (required) - The matter ID.
        * **user_id**: `str` (required) - The user ID of the user to add to the matter.

        **Returns**: An [`AddCustodianResponse`][addcustodianresponse-model] object with the membership for the given matter and user.
        """
        return AddCustodianResponse.parse_response(
            self._parent.session.post(
                f"/v1/legal-hold/matters/{matter_id}/custodians",
                json={"userId": user_id},
            )
        )

    def get_matters_page(
        self,
        creator_user_id: str = None,
        active: bool = None,
        name: str = None,
        page_num: int = None,
        page_size: int = None,
    ) -> MattersPage:
        """Get a page of matters

        **Parameters:**

        * **creator_user_id**: `str` - Find legal hold matters that were created by the user with this unique identifier.
        * **active**: `bool` - When true, return only active matters. When false, return inactive legal hold matters. Defaults to returning all matters.
        * **name**: `str` - Find legal hold matters whose 'name' either equals or partially contains this value.
        * **page_num**: `int` - The page number to request.
        * **page_size**: `int` - The page size to request

        **Returns**: A [`MattersPage`][matterspage-model] object with the page of matters.
        """
        return MattersPage.parse_response(
            self._parent.session.get(
                "/v1/legal-hold/matters",
                params={
                    "creatorUserId": creator_user_id,
                    "active": active,
                    "name": name,
                    "page": page_num,
                    "pageSize": page_size,
                },
            )
        )

    def iter_all_matters(
        self,
        creator_user_id: str = None,
        active: bool = None,
        name: str = None,
        page_size: int = None,
    ) -> Iterator[Matter]:
        """Get all matters

        **Parameters:**

        * **creator_user_id**: `str` - Find legal hold matters that were created by the user with this unique identifier.
        * **active**: `bool` - When true, return only active matters. When false, return inactive legal hold matters. Defaults to returning all matters.
        * **name**: `str` - Find legal hold matters whose 'name' either equals or partially contains this value.
        * **page_num**: `int` - The page number to request.
        * **page_size**: `int` - The page size to request

        **Returns**: A generator object that yields [`Matter`][matter-model] objects with the details of each matter.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_matters_page(
                creator_user_id=creator_user_id,
                active=active,
                name=name,
                page_num=page_num,
                page_size=page_size,
            )
            yield from page.matters
            if len(page.matters) < page_size:
                break

    def create_matter(
        self, policy_id: str, name: str, description: str = None, notes: str = None
    ) -> Matter:
        """Create a matter.

        **Parameters:**

        * **policy_id**: `str` (Required) - The policy ID to be used by the created matter.
        * **name**: `str` (Required) - The name of the matter to be created.
        * **description**: `str` - The description of the matter to be created.
        * **notes**: `str` - The notes for the matter to be created.

        **Returns**: A [`Matter`][matter-model] object with details of the created matter.
        """
        return Matter.parse_response(
            self._parent.session.post(
                "/v1/legal-hold/matters",
                json={
                    "policyId": policy_id,
                    "name": name,
                    "description": description,
                    "notes": notes,
                },
            )
        )

    def deactivate_matter(self, matter_id: str) -> Response:
        """
        Deactivate a matter.

        **Parameters:**

        * **matter_id**: `str` (required) - The unique ID for the matter.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.post(
            f"/v1/legal-hold/matters/{matter_id}/deactivate"
        )

    def remove_custodian(self, matter_id: str, user_id: str) -> Response:
        """
        Remove a custodian from a matter.

        **Parameters:**

        * **matter_id**: `str` (required) - The unique ID for the matter.
        * **user_id**: `str` (required) - The unique ID of the user.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.post(
            f"/v1/legal-hold/matters/{matter_id}/custodians/remove",
            json={"userId": user_id},
        )

    def get_matter(self, matter_id) -> Matter:
        """Get a matter

        **Parameters:**

        * **matter_id**: `str` (required) - The ID of the matter.

        **Returns**: A [`Matter`][matter-model] object with matter details.
        """
        return Matter.parse_response(
            self._parent.session.get(f"/v1/legal-hold/matters/{matter_id}")
        )

    def reactivate_matter(self, matter_id: str) -> Response:
        """
        Reactivate a matter.

        **Parameters:**

        * **matter_id**: `str` (required) - The unique ID for the matter.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.post(
            f"/v1/legal-hold/matters/{matter_id}/reactivate"
        )

    def get_policies_page(
        self, page_num: int = None, page_size: int = None
    ) -> LegalHoldPolicyPage:
        """Get a page of policies

        **Parameters:**

        * **page_num**: `int` - The page number to request.
        * **page_size**: `int` - The page size to request

        **Returns**: A [`LegalHoldPolicyPage`][legalholdpolicypage-model] object with the page of policies.
        """
        return LegalHoldPolicyPage.parse_response(
            self._parent.session.get(
                "/v1/legal-hold/policies",
                params={"page": page_num, "pageSize": page_size},
            )
        )

    def iter_all_policies(self, page_size: int = None) -> Iterator[LegalHoldPolicy]:
        """Iterate through all policies

        **Parameters:**

        * **page_size**: `int` - The page size to request

        **Returns**: A generator that yields [`LegalHoldPolicy`][legalholdpolicy-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_policies_page(page_num=page_num, page_size=page_size)
            yield from page.policies
            if len(page.policies) < page_size:
                break

    def create_policy(self, name: str) -> LegalHoldPolicy:
        """Create a legal hold policy.

        **Parameters:**

        * **name**: `str` (Required) - The name of the policy to create.

        **Returns**: A [`LegalHoldPolicy`][legalholdpolicy-model] object with details of the created policy.
        """
        return LegalHoldPolicy.parse_response(
            self._parent.session.post("/v1/legal-hold/policies", json={"name": name})
        )

    def get_policy(self, policy_id: str) -> LegalHoldPolicy:
        """Get details of a legal hold policy.

        **Parameters:**

        * **policy_id**: `str` (Required) - The unique ID of the policy.

        **Returns**: A [`LegalHoldPolicy`][legalholdpolicy-model] object with details of the policy.
        """
        return LegalHoldPolicy.parse_response(
            self._parent.session.get(f"/v1/legal-hold/policies/{policy_id}")
        )



from _incydr_sdk.enums.trusted_activities import ActivityType
from _incydr_sdk.enums.trusted_activities import CloudShareApps
from _incydr_sdk.enums.trusted_activities import CloudSyncApps
from _incydr_sdk.enums.trusted_activities import EmailServices
from _incydr_sdk.enums.trusted_activities import PrincipalType
from _incydr_sdk.enums.trusted_activities import SortKeys

__all__ = [
    "ActivityType",
    "CloudSyncApps",
    "CloudShareApps",
    "EmailServices",
    "PrincipalType",
    "SortKeys",
]



import os

import click
from rich.console import Console

from _incydr_cli.utils import get_user_project_path


ERROR_LOG_FILE_NAME = "incydr_cli.log"
LOG_LEVEL_DEFAULT = "WARNING"
LOG_FILE_DEFAULT = str(os.path.join(get_user_project_path("log"), ERROR_LOG_FILE_NAME))
LOG_STDERR_DEFAULT = "FALSE"

console = Console()


def log_level_callback(ctx, param, value):
    """callback to update log_level environment variable"""
    if not value:
        if not os.environ.get("INCYDR_LOG_LEVEL"):
            os.environ.update({"INCYDR_LOG_LEVEL": LOG_LEVEL_DEFAULT})
        return
    os.environ.update({"INCYDR_LOG_LEVEL": value})
    return value


def log_file_callback(ctx, param, value):
    """callback to update log_file environment variable"""
    if not value:
        if not os.environ.get("INCYDR_LOG_FILE"):
            os.environ.update({"INCYDR_LOG_FILE": LOG_FILE_DEFAULT})
        return
    os.environ.update({"INCYDR_LOG_FILE": value})
    return value


def log_stderr_callback(ctx, param, value):
    """callback to update log_stderr environment variable"""
    if not value:
        if not os.environ.get("INCYDR_LOG_STDERR"):
            os.environ.update({"INCYDR_LOG_STDERR": LOG_STDERR_DEFAULT})
        return value
    os.environ.update({"INCYDR_LOG_STDERR": "TRUE"})
    return value


log_level_option = click.option(
    "--log-level",
    help="Set level for Incydr client logging.",
    callback=log_level_callback,
    expose_value=False,
)

log_file_option = click.option(
    "--log-file",
    help="Specify file path to write log output to.",
    callback=log_file_callback,
    expose_value=False,
)

log_stderr_option = click.option(
    "--log-stderr",
    "log_stderr",
    help="Enable logging to stderr.",
    default=False,
    is_flag=True,
    callback=log_stderr_callback,
    expose_value=False,
)


def logging_options(f):
    f = log_level_option(f)
    f = log_file_option(f)
    f = log_stderr_option(f)
    return f



from typing import Optional

from pydantic import ConfigDict
from pydantic import Field

from _incydr_sdk.core.models import ResponseModel


class Customer(ResponseModel):
    """
    A model providing details of an Incydr customer account.

    **Fields**:

    * **name**: `str` The Code42 account name.
    * **registration_key**: `str` The Code42 registration key (primarily for licensing purposes).
    * **tenant_id**: `str` The unique identifier for the account within Code42.
    """

    name: Optional[str] = Field(frozen=True)
    registration_key: Optional[str] = Field(frozen=True, alias="registrationKey")
    tenant_id: Optional[str] = Field(frozen=True, alias="tenantId")
    model_config = ConfigDict(validate_assignment=True)



from __future__ import annotations

from _incydr_sdk.enums import _Enum


class ProblemType(_Enum):
    """
    Potential issues when running a query on alerts.
    """

    ILLEGAL_VALUE = "IllegalValue"
    ILLEGAL_OPERATOR = "IllegalOperator"
    MUST_BE_EMAIL_ADDRESS = "MustBeEmailAddress"
    MAX_LENGTH_EXCEEDED = "MaxLengthExceeded"
    INVALID_PAGE_SIZE = "InvalidPageSize"
    INVALID_PAGE_NUMBER = "InvalidPageNumber"
    MISSING_VALUE = "MissingValue"
    MISSING_GROUPS = "MissingGroups"
    MISSING_FILTERS = "MissingFilters"
    MAX_FILTERS_EXCEEDED = "MaxFiltersExceeded"
    SEARCH_FAILED = "SearchFailed"



from _incydr_sdk.enums import _Enum


class ActivityType(_Enum):
    DOMAIN = "DOMAIN"
    ACCOUNT_NAME = "ACCOUNT_NAME"
    URL_PATH = "URL_PATH"
    SLACK = "SLACK"
    CLOUD_SHARE = "CLOUD_SHARE"
    CLOUD_SYNC = "CLOUD_SYNC"
    EMAIL = "EMAIL"
    FILE_UPLOAD = "FILE_UPLOAD"
    GIT_PUSH = "GIT_PUSH"
    GIT_REPOSITORY_URI = "GIT_REPOSITORY_URI"


class CloudSyncApps(_Enum):
    BOX = "BOX"
    GOOGLE_DRIVE = "GOOGLE_DRIVE"
    ICLOUD = "ICLOUD"
    ONE_DRIVE = "ONE_DRIVE"


class CloudShareApps(_Enum):
    BOX = "BOX"
    GOOGLE_DRIVE = "GOOGLE_DRIVE"
    ONE_DRIVE = "ONE_DRIVE"


class EmailServices(_Enum):
    GMAIL = "GMAIL"
    OFFICE_365 = "OFFICE_365"


class Name(_Enum):
    DEFAULT = "DEFAULT"
    DROPBOX = "DROPBOX"
    GOOGLE_DRIVE = "GOOGLE_DRIVE"
    ICLOUD = "ICLOUD"
    ONE_DRIVE = "ONE_DRIVE"
    GMAIL = "GMAIL"
    OFFICE_365 = "OFFICE_365"
    BOX = "BOX"


class PrincipalType(_Enum):
    USER = "USER"
    API_KEY = "API_KEY"
    DEVICE = "DEVICE"
    SERVICE = "SERVICE"


class SortKeys(_Enum):
    ACTIVITY_ID = "ACTIVITY_ID"
    DESCRIPTION = "DESCRIPTION"
    TYPE = "TYPE"
    UPDATED_BY_PRINCIPAL_NAME = "UPDATED_BY_PRINCIPAL_NAME"
    UPDATE_TIME = "UPDATE_TIME"
    VALUE = "VALUE"



from itertools import count
from typing import Iterator
from typing import List
from typing import Union

from pydantic import parse_obj_as
from requests import HTTPError
from requests import Response

from _incydr_sdk.alert_rules.models.request import GetRulesRequest
from _incydr_sdk.alert_rules.models.response import RuleDetails
from _incydr_sdk.alert_rules.models.response import RuleUsersList
from _incydr_sdk.exceptions import IncydrException


class MissingUsernameCriterionError(IncydrException):
    """An error raised if an alert rule has no username filter."""

    def __init__(self, rule_id):
        super().__init__(
            f"Missing Username Criterion Error: Rule '{rule_id}' has no username filter."
        )


class AlertRulesClient:
    def __init__(self, parent):
        self._parent = parent
        self._v2 = None

    @property
    def v2(self):
        if self._v2 is None:
            self._v2 = AlertRulesV2(self._parent)
        return self._v2


class AlertRulesV2:
    """
    Client for `/v2/alert-rules` and `/v2/alerts/rules` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.alert_rules.v2.get_page()
    """

    def __init__(self, parent):
        self._parent = parent

    def get_page(
        self, page_num: int = 0, page_size: int = None, watchlist_id: str = None
    ) -> List[RuleDetails]:
        """
        Get a page of alert rules.

        **Parameters**:

        * **page_num**: `int` - Page number for results, starting at 0.
        * **page_size**: `int` - Max number of results to return for a page.
        * **watchlist_id**: `str` - Watchlist ID to filter for alert rules that are associated with this watchlist.

        **Returns**: A list of [`RuleDetails`][ruledetails-model] objects.
        """

        request = GetRulesRequest(
            PageNumber=page_num,
            PageSize=page_size or self._parent.settings.page_size,
            WatchlistId=watchlist_id,
        )
        response = self._parent.session.get("/v2/alert-rules", params=request.dict())
        return parse_obj_as(List[RuleDetails], response.json())

    def iter_all(
        self, page_size: int = None, watchlist_id: str = None
    ) -> Iterator[RuleDetails]:
        """
        Iterate over all alert rules.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`RuleDetails`][ruledetails-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(0):
            page = self.get_page(
                page_num=page_num, page_size=page_size, watchlist_id=watchlist_id
            )
            yield from page
            if len(page) < page_size:
                break

    def get_rule(self, rule_id: str) -> RuleDetails:
        """
        Get a single alert rule.

        **Parameters**:

        * **rule_id**: `str` (required) - A rule ID.

         **Returns**: A [`RuleDetails`][ruledetails-model] object that contains the details for an alert rule.
        """

        response = self._parent.session.get(f"/v2/alert-rules/{rule_id}")
        return RuleDetails.parse_response(response)

    def enable_rules(self, rule_ids: Union[str, List[str]]) -> Response:
        """
        Enable a single rule or a list of alert rules.

        **Parameters**:

        * **rule_ids**: `str`, `List[str]` (required) - A single rule ID or a list of rule IDs.

        **Returns**: A `requests.Response` indicating success.
        """
        if isinstance(rule_ids, str):
            return self._parent.session.post(url=f"/v2/alert-rules/{rule_ids}/enable")
        else:
            return self._parent.session.post(
                url="/v2/alert-rules/enable", json=rule_ids
            )

    def disable_rules(self, rule_ids: Union[str, List[str]]) -> Response:
        """
        Disable a single rule or list of alert rules.

        **Parameters**:

        * **rule_ids**: `str`, `List[str]` (required) - A single rule ID or a list of rule IDs.

        **Returns**: A `requests.Response` indicating success.
        """
        if isinstance(rule_ids, str):
            return self._parent.session.post(url=f"/v2/alert-rules/{rule_ids}/disable")
        else:
            return self._parent.session.post(
                url="/v2/alert-rules/disable", json=rule_ids
            )

    def remove_all_users(self, rule_id: str) -> Response:
        """
        Remove all users from a rule.

        **Parameters**:

        * **rule_id**: `str` (required) - The ID of the rule to update.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.delete(url=f"/v2/alert-rules/{rule_id}/users")

    def get_users(self, rule_id: str) -> RuleUsersList:
        """
        Get all users assigned to a rule.

        Raises a `MissingUsernameCriterionError` if the rule doesn't have a username filter.

        **Parameters**:

        * **rule_id**: `str` (required) - The ID of the rule.

        **Returns**: A [`RuleUsersList`][ruleuserslist-model] model.
        """
        try:
            response = self._parent.session.get(url=f"/v2/alert-rules/{rule_id}/users")
        except HTTPError as e:
            # This endpoint doesn't have query params or a request body, so it should
            # only return a 400 if there is not username filter for the rule.
            if e.response.status_code == 400:
                raise MissingUsernameCriterionError(rule_id)
            raise e
        return RuleUsersList.parse_response(response)



from _incydr_sdk.actors.models import Actor
from _incydr_sdk.actors.models import ActorFamily
from _incydr_sdk.actors.models import ActorsPage
from _incydr_sdk.agents.models import Agent
from _incydr_sdk.agents.models import AgentsPage
from _incydr_sdk.alert_rules.models.response import RuleDetails
from _incydr_sdk.alert_rules.models.response import RuleUser
from _incydr_sdk.alert_rules.models.response import RuleUsersList
from _incydr_sdk.alerts.models.alert import AlertDetails
from _incydr_sdk.alerts.models.alert import AlertSummary
from _incydr_sdk.alerts.models.response import AlertQueryPage
from _incydr_sdk.audit_log.models import AuditEventsPage
from _incydr_sdk.cases.models import Case
from _incydr_sdk.cases.models import CaseFileEvents
from _incydr_sdk.cases.models import CasesPage
from _incydr_sdk.customer.models import Customer
from _incydr_sdk.departments.models import DepartmentsPage
from _incydr_sdk.devices.models import Device
from _incydr_sdk.devices.models import DevicesPage
from _incydr_sdk.directory_groups.models import DirectoryGroup
from _incydr_sdk.directory_groups.models import DirectoryGroupsPage
from _incydr_sdk.file_events.models.event import FileEventV2
from _incydr_sdk.file_events.models.event import User
from _incydr_sdk.file_events.models.response import FileEventsPage
from _incydr_sdk.file_events.models.response import SavedSearch
from _incydr_sdk.risk_profiles.models import RiskProfile
from _incydr_sdk.risk_profiles.models import RiskProfilesPage
from _incydr_sdk.sessions.models.response import Session
from _incydr_sdk.sessions.models.response import SessionsPage
from _incydr_sdk.trusted_activities.models import TrustedActivitiesPage
from _incydr_sdk.trusted_activities.models import TrustedActivity
from _incydr_sdk.users.models import Role
from _incydr_sdk.users.models import UpdateRolesResponse
from _incydr_sdk.users.models import UserRole
from _incydr_sdk.users.models import UsersPage
from _incydr_sdk.watchlists.models.responses import ExcludedActorsList
from _incydr_sdk.watchlists.models.responses import ExcludedUsersList
from _incydr_sdk.watchlists.models.responses import IncludedActorsList
from _incydr_sdk.watchlists.models.responses import IncludedDepartment
from _incydr_sdk.watchlists.models.responses import IncludedDepartmentsList
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroup
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroupsList
from _incydr_sdk.watchlists.models.responses import IncludedUsersList
from _incydr_sdk.watchlists.models.responses import Watchlist
from _incydr_sdk.watchlists.models.responses import WatchlistActor
from _incydr_sdk.watchlists.models.responses import WatchlistMembersList
from _incydr_sdk.watchlists.models.responses import WatchlistMembersListV2
from _incydr_sdk.watchlists.models.responses import WatchlistsPage
from _incydr_sdk.watchlists.models.responses import WatchlistsPageV2
from _incydr_sdk.watchlists.models.responses import WatchlistUser
from _incydr_sdk.watchlists.models.responses import WatchlistV2


__all__ = [
    "Actor",
    "ActorFamily",
    "ActorsPage",
    "Agent",
    "AgentsPage",
    "AlertDetails",
    "AlertSummary",
    "AlertQueryPage",
    "Case",
    "CaseFileEvents",
    "CasesPage",
    "Customer",
    "Device",
    "DevicesPage",
    "SavedSearch",
    "FileEventsPage",
    "FileEventV2",
    "User",
    "UsersPage",
    "UserRole",
    "Role",
    "UpdateRolesResponse",
    "RuleUsersList",
    "RuleUser",
    "RuleDetails",
    "Session",
    "SessionsPage",
    "TrustedActivity",
    "TrustedActivitiesPage",
    "DepartmentsPage",
    "DirectoryGroupsPage",
    "DirectoryGroup",
    "WatchlistV2",
    "Watchlist",
    "WatchlistsPageV2",
    "WatchlistsPage",
    "WatchlistMembersListV2",
    "WatchlistMembersList",
    "IncludedActorsList",
    "ExcludedActorsList",
    "ExcludedUsersList",
    "IncludedUsersList",
    "WatchlistActor",
    "WatchlistUser",
    "IncludedDepartmentsList",
    "IncludedDepartment",
    "IncludedDirectoryGroupsList",
    "IncludedDirectoryGroup",
    "AuditEventsPage",
    "RiskProfilesPage",
    "RiskProfile",
]


__locals = locals()
for __name in __all__:
    if not __name.startswith("__"):
        setattr(__locals[__name], "__module__", "incydr.models")  # noqa



import re

from requests import Response


def get_filename_from_content_disposition(response: Response, fallback=None) -> str:
    if "content-disposition" in response.headers:
        match = re.search("filename=(.*)", response.headers["content-disposition"])
        if match:
            return match.group(1)
    return fallback



import itertools
from itertools import count
from typing import Iterator
from typing import List
from typing import Union
from warnings import warn

import requests
from pydantic import parse_obj_as

from .models.alert import AlertSummary
from .models.request import AddNoteRequest
from .models.request import AlertDetailsRequest
from .models.request import UpdateAlertStateRequest
from .models.response import AlertDetails
from .models.response import AlertQueryPage
from _incydr_sdk.enums.alerts import AlertState
from _incydr_sdk.queries.alerts import AlertQuery


class AlertsV1:
    """
    Client for `/v1/alerts` endpoints.

    This client is deprecated. Use the Sessions client instead.

    Usage example:

        >>> import incydr
        >>> from incydr.enums.alerts import AlertState
        >>> client = incydr.Client(**kwargs)
        >>> client.alerts.v1.change_state("<alert_id>", AlertState.RESOLVED)
    """

    def __init__(self, parent):
        self._parent = parent

    def search(self, query: AlertQuery) -> AlertQueryPage:
        """
        Search for alerts.

        **Parameters**:

        * **query**: `AlertQuery` (required) - The query object to filter alerts by different fields.

        **Returns**: An [`AlertQueryPage`][alertquerypage-model] object.
        """
        warn(
            "Alerts are deprecated. Replaced by Sessions.",
            DeprecationWarning,
            stacklevel=2,
        )

        if not isinstance(query, AlertQuery):
            raise ValueError("query must be an `incydr.AlertQuery` object.")
        query.tenant_id = self._parent.tenant_id
        response = self._parent.session.post(
            "/v1/alerts/query-alerts", json=query.dict()
        )
        return AlertQueryPage.parse_response(response)

    def iter_all(self, query: AlertQuery) -> Iterator[AlertSummary]:
        """
        Retrieve all alerts for a given query, automatically retrieving multiple pages if they exist.

        * **query**: `AlertQuery` (required) - The query object used to filter alerts by different fields.

        **Returns**: A generator yielding individual [`AlertSummary`][alertsummary-model] objects.
        """
        warn(
            "Alerts are deprecated. Replaced by Sessions.",
            DeprecationWarning,
            stacklevel=2,
        )

        if not isinstance(query, AlertQuery):
            raise ValueError("query must be an `incydr.AlertQuery` object.")
        query.tenant_id = self._parent.tenant_id
        for page_num in count(0):
            query.page_num = page_num
            response = self._parent.session.post(
                "/v1/alerts/query-alerts", json=query.dict()
            )
            page = AlertQueryPage.parse_response(response)
            yield from page.alerts
            if len(page.alerts) < query.page_size:
                break

    def get_details(self, alert_ids: Union[str, List[str]]) -> List[AlertDetails]:
        """
        Get full details for a set of alerts.

        The `query-details` endpoint accepts a maximum of 100 ids per request, if `alert_ids` is > 100, multiple
        requests will be made and results will be combined into a single list.

        **Parameters**:

        * **alert_ids**: `str | List[str]` (required) - Single alertId or list of alertId strings.

        **Returns**: A list of [`AlertDetails`][alertdetails-model] objects.
        """
        warn(
            "Alerts are deprecated. Replaced by Sessions.",
            DeprecationWarning,
            stacklevel=2,
        )

        if isinstance(alert_ids, str):
            alert_ids = [alert_ids]
        results = []
        ids = iter(alert_ids)
        while True:
            chunk = list(itertools.islice(ids, 100))
            if not chunk:
                break
            else:
                data = AlertDetailsRequest(alertIds=chunk)
            response = self._parent.session.post(
                "/v1/alerts/query-details", json=data.dict(by_alias=True)
            )
            results.extend(parse_obj_as(List[AlertDetails], response.json()["alerts"]))
        return results

    def add_note(self, alert_id: str, note: str) -> requests.Response:
        """
        Add a note to an alert.

        **Parameters**:

        * **alert_id**: `List[str]` (required) -

        **Returns**: A `Response` object indicating success.
        """
        warn(
            "Alerts are deprecated. Replaced by Sessions.",
            DeprecationWarning,
            stacklevel=2,
        )

        data = AddNoteRequest(
            alertId=alert_id,
            note=note,
            tenantId=self._parent.tenant_id,
        )
        return self._parent.session.post(
            "/v1/alerts/add-note", json=data.dict(by_alias=True)
        )

    def change_state(
        self, alert_ids: Union[str, List[str]], state: AlertState, note: str = None
    ) -> requests.Response:
        """
        Change the state of a set of alerts (and optionally add note indicating reason for change in the same request).

        **Parameters**:

        * **alert_id**: `str | List[str]` (required) - ID or list of IDs of the alert(s) to update.
        * **state**: [`AlertState`][alert-state] (required) - State to set alert(s) to.
        * **note**: `str` - Optional note text.

        **Returns**: A `Response` object indicating success.
        """
        warn(
            "Alerts are deprecated. Replaced by Sessions.",
            DeprecationWarning,
            stacklevel=2,
        )

        if isinstance(alert_ids, str):
            alert_ids = [alert_ids]
        data = UpdateAlertStateRequest(
            tenantId=self._parent.tenant_id,
            alertIds=alert_ids,
            state=state,
            note=note,
        )
        return self._parent.session.post(
            "/v1/alerts/update-state", json=data.dict(by_alias=True)
        )


class AlertsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = AlertsV1(self._parent)
        return self._v1



from datetime import datetime
from typing import List
from typing import Optional

from pydantic import Field
from pydantic import validator

from _incydr_sdk.core.models import ResponseModel
from _incydr_sdk.queries.utils import parse_str_to_dt


# V1 org endpoints are inconsistent in their date formatting, so we need
# to use a parse method here.
def parse_datetime_validator(cls, value):
    if value:
        return parse_str_to_dt(value)
    else:
        return None


class Org(ResponseModel):
    """
    A model representing an organization.

    **Fields**:

    * **org_guid**: `str` - The globally unique ID of this org.
    * **org_name**: `str` - The name of this org.
    * **org_ext_ref**: `str` - Optional external reference information, such as a serial number, asset tag, employee ID, or help desk issue ID.
    * **notes**: `str` - The notes for this org. Intended for optional additional descriptive information.
    * **parent_org_guid**: `str` - The globally unique ID of the parent org.
    * **active**: `bool` - Whether or not the org is currently active.
    * **creation_date**: `datetime` - Date and time this org was created.
    * **modification_date**: `datetime` - Date and time this org was last modified.
    * **deactivation_date**: `datetime` - Date and time this org was deactivated. Blank if org is active.
    * **registration_key**: `str` - The registration key for the org.
    * **user_count**: `int` - The count of users within this org.
    * **computer_count**: `int` - The count of computers within this org.
    * **org_count**: `int` - The count of child orgs for this org.
    """

    org_guid: Optional[str] = Field(
        None,
        alias="orgGuid",
        description="The globally unique ID of this org.",
    )
    org_name: Optional[str] = Field(
        None,
        alias="orgName",
        description="The name of this org.",
    )
    org_ext_ref: Optional[str] = Field(
        None,
        alias="orgExtRef",
        description="Optional external reference information, such as a serial number, asset tag, employee ID, or help desk issue ID.",
    )
    notes: Optional[str] = Field(
        None,
        alias="notes",
        description="The notes for this org. Intended for optional additional descriptive information.",
    )
    parent_org_guid: Optional[str] = Field(
        None,
        alias="parentOrgGuid",
        description="The globally unique ID of the parent org.",
    )
    active: Optional[bool] = Field(
        None,
        description="Whether or not the org is currently active.",
    )
    creation_date: Optional[datetime] = Field(
        None,
        alias="creationDate",
        description="Date and time this org was created.",
    )
    modification_date: Optional[datetime] = Field(
        None,
        alias="modificationDate",
        description="Date and time this org was last modified.",
    )
    deactivation_date: Optional[datetime] = Field(
        None,
        alias="deactivationDate",
        description="Date and time this org was deactivated. Blank if org is active.",
    )
    registration_key: Optional[str] = Field(
        None,
        alias="registrationKey",
        description="The registration key for the org.",
    )
    user_count: Optional[int] = Field(
        None,
        alias="userCount",
        description="The count of users within this org.",
    )
    computer_count: Optional[int] = Field(
        None,
        alias="computerCount",
        description="The count of computers within this org.",
    )
    org_count: Optional[int] = Field(
        None,
        alias="orgCount",
        description="The count of child orgs for this org.",
    )
    _creation_date = validator("creation_date", allow_reuse=True, pre=True)(
        parse_datetime_validator
    )
    _modification_date = validator("modification_date", allow_reuse=True, pre=True)(
        parse_datetime_validator
    )
    _deactivation_date = validator("deactivation_date", allow_reuse=True, pre=True)(
        parse_datetime_validator
    )


class OrgsList(ResponseModel):
    """
    A model representing a list of `Org` objects.

    **Fields**:

    * **orgs**: `List[Org]` - The list of orgs retrieved from the query.
    * **total_count**: `int` - Total count of orgs found by query.
    """

    orgs: Optional[List[Org]] = Field(None, description="A list of orgs")
    total_count: Optional[int] = Field(
        None, alias="totalCount", description="The total number of orgs"
    )



from _incydr_sdk.enums.alerts import AlertSeverity
from _incydr_sdk.enums.alerts import AlertState
from _incydr_sdk.enums.alerts import AlertTerm
from _incydr_sdk.enums.alerts import RiskSeverity

__all__ = ["AlertState", "AlertSeverity", "AlertTerm", "RiskSeverity"]



import os
from pathlib import Path

import click

from _incydr_cli import logging_options
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_sdk.core.client import Client

path_option = click.option(
    "--path",
    help='The file path where to save the file. The path must include the file name (e.g. "/path/to/my_file.txt"). Defaults to a file named "downloaded_file" in the current directory.',
    default=str(Path(os.getcwd()) / "downloaded_file"),
)


@click.group(cls=IncydrGroup)
@logging_options
def files():
    """Download files by SHA256 hash."""


@files.command(cls=IncydrCommand)
@click.argument("SHA256")
@path_option
@logging_options
def download(sha256: str, path: str):
    """
    Download the file matching the given SHA256 hash to the target path.
    """
    client = Client()
    client.files.v1.download_file_by_sha256(sha256, path)


@files.command(cls=IncydrCommand)
@click.argument("XFC_ID")
@path_option
@logging_options
def download_by_xfc_id(xfc_id: str, path: str):
    """
    Download the file matching the given XFC content ID hash to the target path.
    """
    client = Client()
    client.files.v1.download_file_by_xfc_content_id(
        xfc_content_id=xfc_id, target_path=path
    )



from itertools import count
from typing import Iterator
from typing import List
from typing import Optional
from typing import Union

import requests

from ..enums import SortDirection
from .models import Agent
from .models import AgentsPage
from .models import AgentType
from .models import AgentUpdateRequest
from .models import QueryAgentsRequest
from .models import SortKeys


class AgentsV1:
    """
    Client for `/v1/agents` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.agents.v1.get_page()
    """

    def __init__(self, parent):
        self._parent = parent

    def get_page(
        self,
        active: bool = None,
        agent_type: Optional[AgentType] = None,
        sort_dir: SortDirection = SortDirection.ASC,
        sort_key: SortKeys = SortKeys.NAME,
        page_num: int = 1,
        page_size: int = 500,
        agent_healthy: bool = None,
        agent_health_issue_types: Union[List[str], str] = None,
        user_id: str = None,
    ) -> AgentsPage:
        """
        Get a page of agents.

        Filter results by passing the appropriate parameters:

        **Parameters**:

        * **active**: `bool | None` - When `True`, return only active agents. When `False`, return only deactivated agents. Defaults to `None` (returning both).
        * **agent_type**: `AgentType | str | None` - Return only agents of given type.
        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return per page.
        * **sort_dir**: `SortDirection` - `asc` or `desc`. The direction in which to sort the response based on the corresponding key. Defaults to `asc`.
        * **sort_key**: [`SortKeys`][agents-sort-keys] - Values on which the response will be sorted. Defaults to agent name.
        * **agent_healthy**: `bool | None` - Optionally retrieve agents with this health status. Agents that have no health issue types are considered healthy.
        * **agent_health_issue_types**: `List[str] | str` - Optionally retrieve agents that have (at least) any of the given issue type(s). Health issue types include the following: `NOT_CONNECTING`, `NOT_SENDING_SECURITY_EVENTS`, `SECURITY_INGEST_REJECTED`, `MISSING_MACOS_PERMISSION_FULL_DISK_ACCESS`, `MISSING_MACOS_PERMISSION_ACCESSIBILITY`.
        * **user_id**: `str` - Optionally retrieve only agents associated with this user ID.

        **Returns**: An [`AgentsPage`][agentspage-model] object.
        """
        data = QueryAgentsRequest(
            active=active,
            agentType=agent_type,
            agentHealthy=agent_healthy,
            anyOfAgentHealthIssueTypes=[agent_health_issue_types]
            if isinstance(agent_health_issue_types, str)
            else agent_health_issue_types,
            srtDir=sort_dir,
            srtKey=sort_key,
            pageSize=page_size,
            page=page_num,
            userId=user_id,
        )
        response = self._parent.session.get("/v1/agents", params=data.dict())
        return AgentsPage.parse_response(response)

    def iter_all(
        self,
        active: bool = None,
        agent_type: Optional[AgentType] = None,
        sort_dir: SortDirection = SortDirection.ASC,
        sort_key: SortKeys = SortKeys.NAME,
        page_size: int = 500,
        agent_healthy: bool = None,
        agent_health_issue_types: List[str] = None,
        user_id: str = None,
    ) -> Iterator[Agent]:
        """
        Iterate over all agents.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`Agent`][agent-model] objects.
        """
        for page_num in count(1):
            page = self.get_page(
                active=active,
                agent_type=agent_type,
                agent_healthy=agent_healthy,
                agent_health_issue_types=agent_health_issue_types,
                sort_dir=sort_dir,
                sort_key=sort_key,
                page_num=page_num,
                page_size=page_size,
                user_id=user_id,
            )
            yield from page.agents
            if len(page.agents) < page_size:
                break

    def get_agent(self, agent_id: str) -> Agent:
        """
        Get a single agent.

        **Parameters**:

        * **agent_id**: `str` (required) - The unique ID for the agent.

        **Returns**: An [`Agent`][agent-model] object representing the agent.
        """
        response = self._parent.session.get(f"/v1/agents/{agent_id}")
        return Agent.parse_response(response)

    def update(
        self, agent_id: str, name: str = None, external_reference: str = None
    ) -> requests.Response:
        """
        Update an agent.

        **Parameters:**

        * **agent_id**: `str` (required) - The unique ID for the agent.
        * **name**: `str | None` - The updated name for the agent.
        * **external_reference**: `str | None` - The updated external reference info for the agent.

        **Returns**: A `requests.Response` indicating success.
        """
        data = AgentUpdateRequest(name=name, externalReference=external_reference)
        return self._parent.session.put(f"/v1/agents/{agent_id}", json=data.dict())

    def activate(self, agent_ids: Union[str, List[str]]) -> requests.Response:
        """
        Activate a set of agents.

        **Parameters**:

        * **agent_ids**: `str | List[str]` - An agent ID or list of agent IDs to activate.

        **Returns**: A `requests.Response` indicating success.
        """
        if isinstance(agent_ids, str):
            agent_ids = [agent_ids]
        return self._parent.session.post(
            "/v1/agents/activate", json={"agentIds": agent_ids}
        )

    def deactivate(self, agent_ids: Union[str, List[str]]) -> requests.Response:
        """
        Deactivate a set of agents.

        **Parameters**:

        * **agent_ids**: `str | List[str]` - An agent ID or list of agent IDs to deactivate.

        **Returns**: A `requests.Response` indicating success.
        """
        if isinstance(agent_ids, str):
            agent_ids = [agent_ids]
        return self._parent.session.post(
            "/v1/agents/deactivate", json={"agentIds": agent_ids}
        )


class AgentsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = AgentsV1(self._parent)
        return self._v1



import click

from _incydr_cli.core import incompatible_with
from _incydr_cli.file_readers import FileOrString

# did not include the following fields to filter by:
# Description, HasAuthSignificantWatchlist, Target, LastModifiedAt,
# LastModifiedBy, StateLastModifiedAt, StateLastModifiedBy

AdvancedQueryIncompatible = incompatible_with(["advanced_query"])

start_option = click.option(
    "--start",
    default=None,
    help="The beginning of the date range in which to look for alerts. Filters by alert creation time. "
    "Accepts a date/time in yyyy-MM-dd (UTC) or yyyy-MM-dd HH:MM:SS (UTC+24-hr time) format.",
    cls=incompatible_with(["advanced_query, on"]),
)
end_option = click.option(
    "--end",
    default=None,
    help="The end of the date range in which to look for alerts, argument format options are the same as `--start`.",
    cls=incompatible_with(["advanced_query, on"]),
)
on_option = click.option(
    "--on",
    default=None,
    help="Look for alerts created on the date given, argument forms are the same as `--start`.",
    cls=incompatible_with(["advanced_query, start, end"]),
)

alert_id_option = click.option(
    "--alert-id",
    default=None,
    help="Filter by the unique alert ID.",
    cls=AdvancedQueryIncompatible,
)
type_option = click.option(
    "--type",
    "type_",
    default=None,
    help="Filter by the type of alert. One of [FED_ENDPOINT_EXFILTRATION, FED_CLOUD_SHARE_PERMISSIONS, FED_FILE_TYPE_MISMATCH, FED_FILE_NAME_MATCH, FED_COMPOSITE].",
    cls=AdvancedQueryIncompatible,
)
name_option = click.option(
    "--name",
    default=None,
    help="Filter by the name of the alert.",
    cls=AdvancedQueryIncompatible,
)
actor_option = click.option(
    "--actor",
    default=None,
    help="Filter by the actor (the user who triggered the alert.)",
    cls=AdvancedQueryIncompatible,
)
actor_id_option = click.option(
    "--actor-id",
    default=None,
    help="Filter by the actor ID.",
    cls=AdvancedQueryIncompatible,
)
risk_severity_option = click.option(
    "--risk-severity",
    default=None,
    help="Filter by risk severity.  One of [NO_RISK_INDICATED, LOW, MODERATE, HIGH, CRITICAL].",
    cls=AdvancedQueryIncompatible,
)
state_option = click.option(
    "--state",
    default=None,
    help="Filter by the state of the alert.  One of [OPEN, RESOLVED, IN_PROGRESS, PENDING].",
    cls=AdvancedQueryIncompatible,
)
rule_id_option = click.option(
    "--rule-id",
    default=None,
    help="Filter by the rule ID that corresponds to the rulel which triggered the alert.",
    cls=AdvancedQueryIncompatible,
)
severity_option = click.option(
    "--alert-severity",
    default=None,
    help="Filter by alert severity.  One of [LOW, MEDIUM, HIGH].",
    cls=AdvancedQueryIncompatible,
)

advanced_query_option = click.option(
    "--advanced-query",
    type=FileOrString(),
    default=None,
    help=(
        "A raw JSON alerts query. Useful for when the provided query parameters do not satisfy your requirements.  "
        "Argument can be passed as a string, read from stdin by passing '-', or from a filename if prefixed with '@',"
        "e.g. '--advanced-query @query.json'. WARNING: Using advanced queries is incompatible with other query-"
        "building arguments.  Any additional filter options will be ignored."
    ),
)


def filter_options(f):
    f = start_option(f)
    f = end_option(f)
    f = on_option(f)
    f = alert_id_option(f)
    f = type_option(f)
    f = name_option(f)
    f = actor_option(f)
    f = actor_id_option(f)
    f = risk_severity_option(f)
    f = state_option(f)
    f = rule_id_option(f)
    f = severity_option(f)
    return f



from typing import Optional
from uuid import UUID

import click
import requests
from boltons.iterutils import chunked
from rich.progress import track
from rich.table import Table

from _incydr_cli import console
from _incydr_cli import logging_options
from _incydr_cli import render
from _incydr_cli.cmds.models import UserCSV
from _incydr_cli.cmds.models import UserJSON
from _incydr_cli.cmds.options.output_options import columns_option
from _incydr_cli.cmds.options.output_options import table_format_option
from _incydr_cli.cmds.options.output_options import TableFormat
from _incydr_cli.cmds.options.utils import actor_lookup_callback
from _incydr_cli.cmds.options.utils import user_lookup_callback
from _incydr_cli.cmds.utils import actor_lookup
from _incydr_cli.cmds.utils import deprecation_warning
from _incydr_cli.core import incompatible_with
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_cli.file_readers import FileOrString
from _incydr_cli.render import measure_renderable
from _incydr_cli.render import models_as_table
from _incydr_sdk.core.client import Client
from _incydr_sdk.utils import model_as_card
from _incydr_sdk.watchlists.models.responses import IncludedDepartment
from _incydr_sdk.watchlists.models.responses import IncludedDirectoryGroup
from _incydr_sdk.watchlists.models.responses import WatchlistActor
from _incydr_sdk.watchlists.models.responses import WatchlistUser
from _incydr_sdk.watchlists.models.responses import WatchlistV2

MAX_USER_DISPLAY_COUNT = 25

input_format_option = click.option(
    "--format",
    "-f",
    "format_",
    type=click.Choice(["csv", "json-lines"]),
    default="csv",
    help="Specify format of input file(s): 'csv' or 'json-lines'. Defaults to 'csv'. Multiple input files must all be the same format.",
)


def get_watchlist_id_callback(ctx, param, value):
    if not value:
        return
    try:
        UUID(hex=value)
        return value
    except ValueError:
        # if not an ID value
        client = Client()
        return client.watchlists.v2.get_id_by_name(value)


@click.group(cls=IncydrGroup)
@logging_options
def watchlists():
    """
    View and manage watchlists.

    After creation, Watchlists can be managed by type (ex: `DEPARTING_EMPLOYEE`) or ID. `CUSTOM` watchlists must be managed by title or ID.

    The following values are valid watchlist types:

        * CONTRACT_EMPLOYEE
        * DEPARTING_EMPLOYEE
        * ELEVATED_ACCESS_PRIVILEGES
        * FLIGHT_RISK
        * HIGH_IMPACT_EMPLOYEE
        * NEW_EMPLOYEE
        * PERFORMANCE_CONCERNS
        * POOR_SECURITY_PRACTICES
        * SUSPICIOUS_SYSTEM_ACTIVITY
        * CUSTOM
    """


watchlist_arg = click.argument("watchlist", callback=get_watchlist_id_callback)
csv_option = click.option(
    "--csv", is_flag=True, default=False, help="Users are specified in a CSV file. "
)


@watchlists.command("list", cls=IncydrCommand)
@click.option(
    "--actor",
    default=None,
    help="Filter by watchlists where the actor is a member.  Accepts an actor ID or actor name.  Performs an additional lookup if an actor name is passed",
    callback=actor_lookup_callback,
)
@click.option(
    "--user",
    default=None,
    help="DEPRECATED. Use Actor instead. Filter by watchlists where the user is a member.  Accepts a user ID or a username.  Performs an additional lookup if a username is passed",
    callback=user_lookup_callback,
)
@table_format_option
@columns_option
@logging_options
def list_(
    actor: str = None,
    user: str = None,
    format_: TableFormat = None,
    columns: str = None,
):
    """
    List watchlists.
    """
    if user and not actor:
        actor = user
    client = Client()
    watchlists = client.watchlists.v2.iter_all(actor_id=actor)
    _output_results(watchlists, WatchlistV2, format_, columns)


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@logging_options
def show(
    watchlist: str,
):
    """
    Show details for a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.

    If using `rich`, outputs a summary of watchlist information and membership. This includes the following:

    * included_actors
    * excluded_actors
    * included_departments
    * included_directory_groups

    Lists of actors will be truncated to only display the first 25 members, use the `list-included-actors` and `list-excluded-actors`
    commands respectively to see more details.

    If not using `rich`, outputs watchlist information in JSON without additional membership summary information.
    """
    client = Client()
    watchlist_response = client.watchlists.v2.get(watchlist)

    if not client.settings.use_rich:
        click.echo(watchlist_response.json())
        return

    included_actors = client.watchlists.v2.list_included_actors(
        watchlist
    ).included_actors
    excluded_actors = client.watchlists.v2.list_excluded_actors(
        watchlist
    ).excluded_actors
    departments = client.watchlists.v2.list_departments(watchlist).included_departments
    dir_groups = client.watchlists.v2.list_directory_groups(
        watchlist
    ).included_directory_groups
    t = Table(title=f"{watchlist_response.list_type} Watchlist")
    t.add_column("Stats")

    tables = []
    if included_actors:
        tables.append(
            models_as_table(WatchlistActor, included_actors[:MAX_USER_DISPLAY_COUNT])
        )
        t.add_column("Included Users")
    if excluded_actors:
        tables.append(
            models_as_table(WatchlistActor, excluded_actors[:MAX_USER_DISPLAY_COUNT])
        )
        t.add_column("Excluded Users")
    if departments:
        tables.append(models_as_table(IncludedDepartment, departments))
        t.add_column("Included Departments")
    if dir_groups:
        tables.append(models_as_table(IncludedDirectoryGroup, dir_groups))
        t.add_column("Included Directory Groups")

    t.add_row(model_as_card(watchlist_response), *tables)

    with console.pager():
        # expand console and table so no values get truncated due to size of console, since we're using a pager
        console.width = t.width = measure_renderable(t)
        console.print(t, crop=False, soft_wrap=False, overflow="fold")


@watchlists.command(cls=IncydrCommand)
@click.argument("watchlist-type")
@click.option("--title", help="Required title for a CUSTOM watchlist.")
@click.option("--description", help="Optional description for a CUSTOM watchlist.")
@logging_options
def create(
    watchlist_type: str,
    title: str = None,
    description: str = None,
):
    """
    Create a new watchlist.

    Where `WATCHLIST_TYPE` is of the following:

    * `CONTRACT_EMPLOYEE`
    * `DEPARTING_EMPLOYEE`
    * `ELEVATED_ACCESS_PRIVILEGES`
    * `FLIGHT_RISK`
    * `HIGH_IMPACT_EMPLOYEE`
    * `NEW_EMPLOYEE`
    * `PERFORMANCE_CONCERNS`
    * `POOR_SECURITY_PRACTICES`
    * `SUSPICIOUS_SYSTEM_ACTIVITY`
    * `CUSTOM`

    The `--title` (required) and `--description` (optional) options are exclusively for creating CUSTOM watchlists.
    """
    client = Client()
    watchlist = client.watchlists.v2.create(watchlist_type, title, description)
    console.print(
        f"Successfully created {watchlist.list_type} watchlist with ID: '{watchlist.watchlist_id}'."
    )


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@logging_options
def delete(
    watchlist: str,
):
    """
    Delete a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    client = Client()
    client.watchlists.v2.delete(watchlist)
    console.print(f"Successfully deleted watchlist with ID: '{watchlist}'.")


@watchlists.command(cls=IncydrCommand)
@click.argument("watchlist-id")
@click.option("--title", help="Updated title for a CUSTOM watchlist.")
@click.option("--description", help="Updated description for a CUSTOM watchlist.")
@click.option(
    "--clear-description",
    is_flag=True,
    default=False,
    help="Clear the description on a CUSTOM watchlist.",
    cls=incompatible_with("description"),
)
@logging_options
def update(
    watchlist_id: str,
    title: str = None,
    description: str = None,
    clear_description: str = None,
):
    """
    Update a CUSTOM watchlist.
    """
    if clear_description:
        description = ""

    client = Client()
    client.watchlists.v2.update(watchlist_id, title, description)
    console.print(f"Successfully updated watchlist with ID: '{watchlist_id}'.")


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@click.option(
    "--actors",
    default=None,
    type=FileOrString(),
    help="List of actor IDs or actor names to include on the watchlist. "
    "An additional lookup is performed if an actor name is passed. Argument can be "
    "passed as a comma-delimited string or from a CSV file with a single 'actor' "
    "column if prefixed with '@', e.g. '--actors @actors.csv'.",
)
@click.option(
    "--excluded-actors",
    default=None,
    type=FileOrString(),
    help="List of actor IDs or actor names to exclude from the watchlist. "
    "An additional lookup is performed if an actor name is passed. Argument can be "
    "passed as a comma-delimited string or from a CSV file with a single 'actor' "
    "column if prefixed with '@', e.g. '--excluded-actors @actors.csv'.",
)
@click.option(
    "--departments",
    default=None,
    help="Comma-delimited string of department names to include on the watchlist. "
    "Individual users from the departments will be added as watchlist members, where department information comes "
    "from SCIM or User Directory Sync.",
)
@click.option(
    "--directory-groups",
    default=None,
    help="Comma-delimited string of directory group IDs to include on the watchlist. "
    "Individual users from the directory groups will be added as watchlist members, where group information comes "
    "from SCIM or User Directory Sync.",
)
@click.option(
    "--users",
    default=None,
    type=FileOrString(),
    help="DEPRECATED. Use --actors instead. List of user IDs or usernames to include on the watchlist. "
    "An additional lookup is performed if a username is passed. Argument can be "
    "passed as a comma-delimited string or from a CSV file with a single 'user' "
    "column if prefixed with '@', e.g. '--users @users.csv'.",
)
@click.option(
    "--excluded-users",
    default=None,
    type=FileOrString(),
    help="DEPRECATED. Use --excluded-actors instead. List of user IDs or usernames to exclude from the watchlist. "
    "An additional lookup is performed if a username is passed. Argument can be "
    "passed as a comma-delimited string or from a CSV file with a single 'user' "
    "column if prefixed with '@', e.g. '--users @users.csv'.",
)
@input_format_option
@logging_options
def add(
    watchlist: str,
    actors=None,
    excluded_actors=None,
    departments=None,
    directory_groups=None,
    users=None,
    excluded_users=None,
    format_=None,
):
    """
    Manage watchlist membership by including or excluding individual actors and/or groups.

    Add any of the following members to a watchlist with the corresponding options:

    * actors
    * excluded-actors
    * departments
    * directory-groups

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.

    If adding or excluding more than 100 actors in a single run, the CLI will automatically batch
    requests due to a limit of 100 per request on the backend.
    """
    client = Client()

    if users and not actors:
        actors = users
    if excluded_users and not excluded_actors:
        excluded_actors = excluded_users

    # Add included actors
    if actors:
        actor_ids, errors = _get_actor_ids(client, actors, format_=format_)
        succeeded = 0
        for chunk in chunked(actor_ids, size=100):
            try:
                client.watchlists.v2.add_included_actors(watchlist, chunk)
                console.print(
                    f"Successfully included {len(chunk)} actors on watchlist with ID: '{watchlist}'"
                )
                succeeded += len(chunk)
            except requests.HTTPError as err:
                if "Actor not found" in err.response.text:
                    console.print(
                        "Problem processing batch of actors, will attempt each individually."
                    )
                    chunk_succeeded = 0
                    for actor in chunk:
                        try:
                            client.watchlists.v2.add_included_actors(watchlist, actor)
                            succeeded += 1
                            chunk_succeeded += 1
                        except requests.HTTPError as err:
                            client.settings.logger.error(
                                f"Problem adding actorId={actor} to watchlist={watchlist}: {err.response.text}"
                            )
                            errors.append(actor)
                    console.print(
                        f"Successfully included {chunk_succeeded} users on watchlist with ID: '{watchlist}'"
                    )
                else:
                    raise err
        if errors:
            console.print("[red]The following usernames/user IDs were not found:")
            console.print("\t" + "\n\t".join(errors))

    # Add excluded actors
    if excluded_actors:
        succeeded = 0
        actor_ids, errors = _get_actor_ids(client, excluded_actors, format_=format_)
        for chunk in chunked(actor_ids, size=100):
            try:
                client.watchlists.v2.add_excluded_actors(watchlist, chunk)
                console.print(
                    f"Successfully excluded {len(chunk)} actors from watchlist with ID: '{watchlist}'"
                )
                succeeded += len(chunk)
            except requests.HTTPError as err:
                if "Actor not found" in err.response.text:
                    console.print(
                        "Problem processing batch of actors, will attempt each individually."
                    )
                    chunk_succeeded = 0
                    for actor in chunk:
                        try:
                            client.watchlists.v2.add_excluded_actors(watchlist, actor)
                            succeeded += 1
                            chunk_succeeded += 1
                        except requests.HTTPError as err:
                            client.settings.logger.error(
                                f"Problem excluding actorId={actor} from watchlist={watchlist}: {err.response.text}"
                            )
                            errors.append(actor)
                    console.print(
                        f"Successfully excluded {chunk_succeeded} users from watchlist with ID: '{watchlist}'"
                    )
                else:
                    raise err
        if errors:
            console.print("[red]The following actornames/actor IDs were not found:")
            console.print("\t" + "\n\t".join(errors))

    # Add departments
    if departments:
        client.watchlists.v2.add_departments(
            watchlist, [i.strip() for i in departments.split(",")]
        )
        console.print(
            f"Successfully included departments to watchlist with ID: {watchlist}"
        )

    # Add directory groups
    if directory_groups:
        client.watchlists.v2.add_directory_groups(
            watchlist, [i.strip() for i in directory_groups.split(",")]
        )
        console.print(
            f"Successfully included directory groups to watchlist with ID: {watchlist}"
        )


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@click.option(
    "--actors",
    default=None,
    type=FileOrString(),
    help="List of actor IDs or actor names to remove from the watchlist. "
    "An additional lookup is performed if an actor name is passed. Argument can be "
    "passed as a comma-delimited string or from a CSV file with a single 'actor' "
    "column if prefixed with '@', e.g. '--actors @actors.csv'. "
    "File should have a single 'actor' field.  File format can either be CSV or JSON Lines format, "
    "as specified with the --format option (Default is CSV).",
)
@click.option(
    "--excluded-actors",
    default=None,
    type=FileOrString(),
    help="List of actor IDs or actor names to remove from the watchlist. "
    "An additional lookup is performed if an actor name is passed. Argument can be "
    "passed as a comma-delimited string or from a CSV file with a single 'actor' "
    "column if prefixed with '@', e.g. '--excluded-actors @actors.csv'. "
    "File should have a single 'actor' field.  File format can either be CSV or JSON Lines format, "
    "as specified with the --format option (Default is CSV).",
)
@click.option(
    "--departments",
    default=None,
    help="Comma-delimited string of department names to remove from the watchlist. "
    "Individual users from the departments will be added as watchlist members, where department information comes "
    "from SCIM or User Directory Sync.",
)
@click.option(
    "--directory-groups",
    default=None,
    help="Comma-delimited string of directory group IDs to remove from the watchlist. "
    "Individual users from the directory groups will be added as watchlist members, where group information comes "
    "from SCIM or User Directory Sync.",
)
@click.option(
    "--users",
    default=None,
    type=FileOrString(),
    help="DEPRECATED. Use --actors instead. List of included user IDs or usernames to remove from the watchlist. "
    "An additional lookup is performed if a username is passed.Argument can be "
    "passed as a comma-delimited string or as a file if prefixed with '@', e.g. '--users @users.csv'. "
    "File should have a single 'user' field.  File format can either be CSV or JSON Lines format, "
    "as specified with the --format option (Default is CSV).",
)
@click.option(
    "--excluded-users",
    default=None,
    type=FileOrString(),
    help="DEPRECATED. Use --excluded-actors instead. List of excluded user IDs or usernames to remove from the watchlist. "
    "An additional lookup is performed if a username is passed. Argument can be "
    "passed as a comma-delimited string or as a file if prefixed with '@', e.g. '--users @users.csv'. "
    "File should have a single 'user' field.  File format can either be CSV or JSON Lines format, "
    "as specified with the --format option (Default is CSV).",
)
@input_format_option
@logging_options
def remove(
    watchlist: str,
    actors=None,
    excluded_actors=None,
    departments=None,
    directory_groups=None,
    users=None,
    excluded_users=None,
    format_=None,
):
    """
    Manage watchlist membership by removing individual users and/or groups.

    Remove any of the following members from a watchlist with the corresponding options:

    * users
    * excluded-users
    * departments
    * directory-groups

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.

    If removing more than users or exclusions in a single run, the CLI will automatically batch
    requests due to a limit of 100 per request on the backend.
    """
    client = Client()

    if users and not actors:
        actors = users
    if excluded_users and not excluded_actors:
        excluded_actors = excluded_users

    # Remove included users
    if actors:
        actor_ids, errors = _get_actor_ids(client, actors, format_=format_)
        succeeded = 0
        for chunk in chunked(actor_ids, size=100):
            try:
                client.watchlists.v2.remove_included_actors(watchlist, chunk)
                console.print(
                    f"Successfully removed {len(chunk)} included actors on watchlist with ID: '{watchlist}'"
                )
                succeeded += len(chunk)
            except requests.HTTPError as err:
                if "Actor not found" in err.response.text:
                    console.print(
                        "Problem processing batch of actors, will attempt each individually."
                    )
                    chunk_succeeded = 0
                    for actor in chunk:
                        try:
                            client.watchlists.v2.remove_included_actors(
                                watchlist, actor
                            )
                            succeeded += 1
                            chunk_succeeded += 1
                        except requests.HTTPError as err:
                            client.settings.logger.error(
                                f"Problem removing actorId={actor} from watchlist={watchlist}: {err.response.text}"
                            )
                            errors.append(actor)
                    console.print(
                        f"Successfully removed {chunk_succeeded} actors from watchlist with ID: '{watchlist}'"
                    )
                else:
                    raise err
        if errors:
            console.print("[red]The following actornames/actor IDs were not found:")
            console.print("\t" + "\n\t".join(errors))

    # Remove excluded users
    if excluded_actors:
        actor_ids, errors = _get_actor_ids(client, excluded_actors, format_=format_)
        succeeded = 0
        for chunk in chunked(actor_ids, size=100):
            try:
                client.watchlists.v2.remove_excluded_actors(watchlist, chunk)
                console.print(
                    f"Successfully removed {len(chunk)} excluded actors from watchlist with ID: '{watchlist}'"
                )
                succeeded += len(chunk)
            except requests.HTTPError as err:
                if "User not found" in err.response.text:
                    console.print(
                        "Problem processing batch of actors, will attempt each individually."
                    )
                    chunk_succeeded = 0
                    for actor in chunk:
                        try:
                            client.watchlists.v2.remove_excluded_actors(
                                watchlist, actor
                            )
                            succeeded += 1
                            chunk_succeeded += 1
                        except requests.HTTPError as err:
                            client.settings.logger.error(
                                f"Problem removing excluded actorId={actor} from watchlist={watchlist}: {err.response.text}"
                            )
                            errors.append(actor)
                    console.print(
                        f"Successfully removed {chunk_succeeded} excluded actors from watchlist with ID: '{watchlist}'"
                    )
                else:
                    raise err
            if errors:
                console.print("[red]The following actornames/actor IDs were not found:")
                console.print("\t" + "\n\t".join(errors))

    # Remove departments
    if departments:
        client.watchlists.v2.remove_departments(
            watchlist, [i.strip() for i in departments.split(",")]
        )
        console.print(
            f"Successfully removed departments from watchlist with ID: {watchlist}"
        )

    # Remove directory groups
    if directory_groups:
        client.watchlists.v2.remove_directory_groups(
            watchlist, [i.strip() for i in directory_groups.split(",")]
        )
        console.print(
            f"Successfully removed directory groups from watchlist with ID: {watchlist}"
        )


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@columns_option
@table_format_option
@logging_options
def list_members(
    watchlist: str,
    format_: TableFormat,
    columns: Optional[str],
):
    """
    List members of a watchlist.

    A member may have been added as an included user, or is a member of an included department, etc.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    client = Client()
    members = list(client.watchlists.v2.iter_all_members(watchlist))
    _output_results(members, WatchlistActor, format_, columns)


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@table_format_option
@columns_option
@logging_options
def list_included_actors(
    watchlist: str,
    format_: str,
    columns: Optional[str],
):
    """
    List actors explicitly included on a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    client = Client()
    users = list(client.watchlists.v2.iter_all_included_actors(watchlist))
    _output_results(users, WatchlistActor, format_, columns)


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@table_format_option
@columns_option
@logging_options
def list_excluded_actors(
    watchlist: str,
    format_: TableFormat,
    columns: Optional[str],
):
    """
    List actors excluded from a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    client = Client()
    users = list(client.watchlists.v2.iter_all_excluded_actors(watchlist))
    _output_results(users, WatchlistActor, format_, columns)


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@table_format_option
@columns_option
@logging_options
def list_directory_groups(
    watchlist: str,
    format_: TableFormat,
    columns: Optional[str],
):
    """
    List directory groups included on a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    client = Client()
    groups = list(client.watchlists.v2.iter_all_directory_groups(watchlist))
    _output_results(groups, IncludedDirectoryGroup, format_, columns)


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@table_format_option
@columns_option
@logging_options
def list_departments(
    watchlist: str,
    format_: TableFormat,
    columns: Optional[str],
):
    """
    List departments included on a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    client = Client()
    deps = list(client.watchlists.v2.iter_all_departments(watchlist))
    _output_results(deps, IncludedDepartment, format_, columns)


def _get_actor_ids(client, actors, format_=None):
    ids, errors = [], []
    if isinstance(actors, str):
        for actor in actors.split(","):
            try:
                actor = actor_lookup(client, actor)
                ids.append(actor)
            except ValueError:
                client.settings.logger.error(
                    f"Problem looking up actorId for actor name: {actor}"
                )
                errors.append(actor)
    else:
        if format_ == "csv":
            actors = UserCSV.parse_csv(actors)
        else:
            actors = UserJSON.parse_json_lines(actors)
        for row in track(
            actors,
            description="Reading actors...",
            transient=True,
        ):
            try:
                actor_id = actor_lookup(client, row.user)
                ids.append(actor_id)
            except ValueError:
                client.settings.logger.error(
                    f"Problem looking up actorId for actor name: {row.user}"
                )
                errors.append(row.user)
    return ids, errors


def _output_results(results, model, format_, columns=None):
    if format_ == TableFormat.csv:
        render.csv(model, results, columns=columns, flat=True)
    elif format_ == TableFormat.table:
        render.table(model, results, columns=columns, flat=False)
    elif format_ == TableFormat.json_pretty:
        for item in results:
            console.print_json(item.json())
    else:
        for item in results:
            console.print(item.json(), highlight=False)


# Deprecated 2025-03. Will be removed 2026-03.


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@table_format_option
@columns_option
@logging_options
def list_included_users(
    watchlist: str,
    format_: str,
    columns: Optional[str],
):
    """
    DEPRECATED. List users explicitly included on a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    deprecation_warning("DEPRECATED. Use list_included_actors instead.")
    client = Client()
    users = client.watchlists.v1.list_included_users(watchlist)
    _output_results(users.included_users, WatchlistUser, format_, columns)


@watchlists.command(cls=IncydrCommand)
@watchlist_arg
@table_format_option
@columns_option
@logging_options
def list_excluded_users(
    watchlist: str,
    format_: TableFormat,
    columns: Optional[str],
):
    """
    DEPRECATED. List users excluded from a watchlist.

    WATCHLIST can be specified by watchlist type (ex: `DEPARTING_EMPLOYEE`) or ID.
    `CUSTOM` watchlists must be specified by title or ID.
    """
    deprecation_warning("DEPRECATED. Use list_excluded_actors instead.")

    client = Client()
    users = client.watchlists.v1.list_excluded_users(watchlist)
    _output_results(users.excluded_users, WatchlistUser, format_, columns)



from .models import Customer


class CustomerClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = CustomerV1(self._parent)
        return self._v1


class CustomerV1:
    """
    Client for `/v1/customer` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.customer.v1.get()
    """

    def __init__(self, parent):
        self._parent = parent

    def get(self) -> Customer:
        """
        Get customer account information.

        **Returns**: A [`Customer`][customer-model] object representing account information.

        """
        response = self._parent.session.get("/v1/customer")
        return Customer.parse_response(response)



from __future__ import annotations

from _incydr_sdk.enums import _Enum
from _incydr_sdk.enums.file_events import RiskSeverity  # noqa


class AlertState(_Enum):
    """
    Enum indicating possible alert states.
    """

    OPEN = "OPEN"
    RESOLVED = "RESOLVED"
    IN_PROGRESS = "IN_PROGRESS"
    PENDING = "PENDING"


class RuleType(_Enum):
    """
    Rule type that generates alerts.
    """

    FED_ENDPOINT_EXFILTRATION = "FED_ENDPOINT_EXFILTRATION"
    FED_CLOUD_SHARE_PERMISSIONS = "FED_CLOUD_SHARE_PERMISSIONS"
    FED_FILE_TYPE_MISMATCH = "FED_FILE_TYPE_MISMATCH"
    FED_FILE_NAME_MATCH = "FED_FILE_NAME_MATCH"
    FED_COMPOSITE = "FED_COMPOSITE"


class AlertSeverity(_Enum):
    """
    Possible severity values for an alert.
    """

    LOW = "LOW"
    MEDIUM = "MEDIUM"
    HIGH = "HIGH"


class Operator(_Enum):
    """
    The alert filter operators.
    """

    IS = "IS"
    IS_NOT = "IS_NOT"
    ON_OR_BEFORE = "ON_OR_BEFORE"
    ON_OR_AFTER = "ON_OR_AFTER"
    ON = "ON"
    CONTAINS = "CONTAINS"
    DOES_NOT_CONTAIN = "DOES_NOT_CONTAIN"


class AlertTerm(_Enum):
    ALERT_ID = "AlertId"
    TYPE = "Type"
    NAME = "Name"
    DESCRIPTION = "Description"
    ACTOR = "Actor"
    ACTOR_ID = "ActorId"
    TARGET = "Target"
    RISK_SEVERITY = "RiskSeverity"
    CREATED_AT = "CreatedAt"
    HAS_AUTH_SIGNIFICANT_WATCHLIST = "HasAuthSignificantWatchlist"
    STATE = "State"
    STATE_LAST_MODIFIED_AT = "StateLastModifiedAt"
    STATE_LAST_MODIFIED_BY = "StateLastModifiedBy"
    LAST_MODIFIED_TIME = "LastModifiedTime"
    LAST_MODIFIED_BY = "LastModifiedBy"
    RULE_ID = "RuleId"
    SEVERITY = "AlertSeverity"


class MessagingMethod(_Enum):
    """
    Instructor messaging method for an alert rule.

    Only part of alert rule responses as of 11/28/2022.
    """

    EMAIL = "Email"
    SLACK = "Slack"
    TEAMS = "Teams"



# SPDX-FileCopyrightText: 2022-present Code42 Software <integrations@code42.com>
#
# SPDX-License-Identifier: MIT
from . import enums
from . import models
from _incydr_sdk import exceptions
from _incydr_sdk.__version__ import __version__
from _incydr_sdk.core.client import Client
from _incydr_sdk.queries.alerts import AlertQuery
from _incydr_sdk.queries.file_events import EventQuery

__all__ = [
    "__version__",
    "Client",
    "AlertQuery",
    "EventQuery",
    "models",
    "exceptions",
]

__locals = locals()
for __name in __all__:
    if not __name.startswith("__"):
        setattr(__locals[__name], "__module__", "incydr")  # noqa



from datetime import date
from datetime import datetime
from datetime import timezone
from typing import Union

from dateutil import parser

from _incydr_sdk.exceptions import DateParseError

MICROSECOND_FORMAT = "%Y-%m-%dT%H:%M:%S.%fZ"
DATETIME_STR_FORMAT = "%Y-%m-%d %H:%M:%S"
DATE_STR_FORMAT = "%Y-%m-%d"


def parse_ts_to_date_str(timestamp: Union[str, int, float, datetime, date]):
    """
    Parse int/float/str/datetime timestamp to string milliseconds precision.

    Args:
        timestamp (str or int or float or datetime): A POSIX timestamp.

    **Returns**:
        (str): A str representing the given date. Example output looks like
        '2020-03-25'.
    """
    # convert str/int/float values to datetime
    if isinstance(timestamp, (int, float)):
        timestamp = datetime.fromtimestamp(timestamp, tz=timezone.utc)
    elif isinstance(timestamp, str):
        timestamp = parse_str_to_dt(timestamp)
    timestamp.replace(tzinfo=timezone.utc)
    # parse datetime to string
    return timestamp.strftime(DATE_STR_FORMAT)


def parse_ts_to_ms_str(timestamp: Union[str, int, float, datetime, date]):
    """
    Parse int/float/str/datetime timestamp to string milliseconds precision.

    Args:
        timestamp (str or int or float or datetime): A POSIX timestamp.

    **Returns**:
        (str): A str representing the given timestamp. Example output looks like
        '2020-03-25T15:29:04.465Z'.
    """
    # convert str/int/float values to datetime
    if isinstance(timestamp, (int, float)):
        timestamp = datetime.fromtimestamp(timestamp, tz=timezone.utc)
    elif isinstance(timestamp, str):
        timestamp = parse_str_to_dt(timestamp)
    timestamp.replace(tzinfo=timezone.utc)
    # parse datetime to string
    return f"{timestamp.strftime(MICROSECOND_FORMAT)[:-4]}Z"


def parse_ts_to_posix_ts(timestamp: Union[str, datetime]):
    """
    Parse POSIX timestamp from DATE/DATETIME str or datetime obj.
    """
    dt = timestamp if isinstance(timestamp, datetime) else parse_str_to_dt(timestamp)
    return dt.timestamp()


def parse_ts_to_ms_ts(timestamp: Union[str, datetime]):
    """
    Parse epoch ms timestamp from DATE/DATETIME str or datetime obj.
    """
    return parse_ts_to_posix_ts(timestamp) * 1000


def parse_str_to_dt(timestamp: str):
    try:
        dt = parser.parse(timestamp)
        return dt.replace(tzinfo=timezone.utc)
    except ValueError:
        raise DateParseError(
            timestamp,
            f"DateParseError: Time data '{timestamp}' does not match any known formats.  Ex: {DATETIME_STR_FORMAT}",
        )



try:
    from _incydr_cli.main import incydr
except ModuleNotFoundError:
    err_message = (
        "Missing CLI dependencies. To use Incydr CLI run: pip install 'incydr[cli]'"
    )
    import sys

    if "--python" in sys.argv:
        print(sys.executable)
    else:
        print(
            "Missing CLI dependencies. To use Incydr CLI run: pip install 'incydr[cli]'"
        )
    exit(1)

incydr()



import click
from rich.table import Table

from _incydr_cli import console
from _incydr_cli import logging_options
from _incydr_cli import render
from _incydr_cli.cmds.options.output_options import columns_option
from _incydr_cli.cmds.options.output_options import single_format_option
from _incydr_cli.cmds.options.output_options import SingleFormat
from _incydr_cli.cmds.options.output_options import table_format_option
from _incydr_cli.cmds.options.output_options import TableFormat
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_sdk.core.client import Client
from _incydr_sdk.enums.trusted_activities import ActivityType
from _incydr_sdk.trusted_activities.client import MissingActivityActionGroupsError
from _incydr_sdk.trusted_activities.models import TrustedActivity
from _incydr_sdk.utils import model_as_card


@click.group(cls=IncydrGroup)
@logging_options
def trusted_activities():
    """View and manage trusted activities."""


@trusted_activities.command(cls=IncydrCommand)
@click.argument("activity_id")
@single_format_option
@logging_options
def show(
    activity_id: str,
    format_: SingleFormat,
):
    """
    Show details for a single trusted activity.

    Includes general info on the trusted activity, as well as any 'Activity Action Groups', which specify
    various trusted service configurations (if applicable). For example, a trusted domain may include an activity
    action group indicating `GMAIL` as a trusted email sharing service.
    """
    client = Client()
    trusted_activity = client.trusted_activities.v2.get_trusted_activity(activity_id)
    _output_trusted_activity(
        trusted_activity, format_, use_rich=client.settings.use_rich
    )


@trusted_activities.command("list", cls=IncydrCommand)
@click.option("--activity-type", type=ActivityType, default=None, help="")
@table_format_option
@columns_option
@logging_options
def list_(
    activity_type: ActivityType = None,
    format_: TableFormat = None,
    columns: str = None,
):
    """
    List all trusted activities.
    """
    client = Client()
    activities = client.trusted_activities.v2.iter_all(activity_type=activity_type)
    if format_ == TableFormat.table:
        columns = columns or [
            "activity_id",
            "type",
            "value",
            "description",
            "principal_type",
            "update_time",
            "updated_by_principal_id",
            "updated_by_principal_name",
        ]
        render.table(TrustedActivity, activities, columns=columns)
    elif format_ == TableFormat.csv:
        render.csv(TrustedActivity, activities, columns=columns)
    elif format_ == TableFormat.json_pretty:
        for activity in activities:
            console.print_json(activity.json())
    else:
        for activity in activities:
            click.echo(activity.json())


@trusted_activities.command(cls=IncydrCommand)
@click.argument("activity_id")
@click.option("--type", "type_", default=None)
@click.option("--value", default=None)
@click.option("--description", default=None)
@click.option("--high-value-source", type=bool, default=None)
@single_format_option
@logging_options
def update(
    activity_id: str,
    type_: str = None,
    value: str = None,
    description: str = None,
    high_value_source: bool = None,
    format_: SingleFormat = None,
):
    """
    Update a trusted activity.
    """
    if not any([type_, value, description, (high_value_source is not None)]):
        raise click.UsageError(
            "At least one command option must be provided to update a trusted activity.  Use `trusted-activities update --help` to see available options."
        )
    # left off updating activity-action-groups for now, they're very complex
    client = Client()
    trusted_activity = client.trusted_activities.v2.get_trusted_activity(activity_id)
    if type_:
        trusted_activity.type = type_
    if value:
        trusted_activity.value = value
    if description:
        trusted_activity.description = description
    if high_value_source is not None:
        trusted_activity.is_high_value_source = high_value_source
    updated_activity = client.trusted_activities.v2.update(trusted_activity)
    _output_trusted_activity(updated_activity, format_, client.settings.use_rich)


@trusted_activities.command(cls=IncydrCommand)
@click.argument("activity_id")
@logging_options
def delete(
    activity_id: str,
):
    """
    Delete a trusted activity.
    """
    client = Client()
    client.trusted_activities.v2.delete(activity_id)
    console.print(f"Successfully deleted trusted activity {activity_id}.")


@trusted_activities.group(cls=IncydrGroup)
@logging_options
def add():
    """Add a new trusted activity."""


@add.command("domain", cls=IncydrCommand)
@click.argument("domain")
@click.option("--description", default=None, help="Optional description.")
@click.option(
    "--file-upload",
    is_flag=True,
    default=False,
    help="Trust file upload events to where the tab URL or title includes this domain.",
)
@click.option(
    "--git-push",
    is_flag=True,
    default=False,
    help="Trust git push events to this domain.",
)
@click.option(
    "--cloud-sync",
    "cloud_sync_services",
    type=click.Choice(["BOX", "GOOGLE_DRIVE", "ICLOUD", "ONE_DRIVE"]),
    default=[],
    help="Specify which cloud sync service(s) to trust.",
    multiple=True,
)
@click.option(
    "--cloud-share",
    "cloud_share_services",
    type=click.Choice(["BOX", "GOOGLE_DRIVE", "ONE_DRIVE"]),
    default=[],
    help="Specify which cloud share service(s) to trust.",
    multiple=True,
)
@click.option(
    "--email-share",
    "email_share_services",
    type=click.Choice(["GMAIL", "MICROSOFT_365"]),
    default=[],
    help="Specify which email share service(s) to trust.",
    multiple=True,
)
@single_format_option
@logging_options
def domain_(
    domain: str,
    description: str = None,
    file_upload: bool = False,
    git_push: bool = False,
    cloud_sync_services: str = None,
    cloud_share_services: str = None,
    email_share_services: str = None,
    format_: SingleFormat = None,
):
    """
    Trust activity across an entire `DOMAIN` (ex: `my-domain.com`).

    The following activities can be configured:

    * `--file-upload` - Trust file uploads to this domain.  Defaults to false.
    * `--git-push` - Trust git push events to this domain.  Defaults to false.
    * `--cloud-sync-services` [`BOX|GOOGLE_DRIVE|ICLOUD|ONE_DRIVE`] - Trust cloud sync activity from the specified service(s) if the username signed into the sync app is on this domain.
        If you want to only trust activity for a specific corporate account, add a trusted account name instead.
    * `--cloud-share-services` [`BOX|GOOGLE_DRIVE|ONE_DRIVE`] - Trust cloud share activity from the specified service(s) if the user its shared with is on this domain.
        You must have a cloud connector configured for your tenant to support this trusted action.
    * `--email-share-services` [`GMAIL|MICROSOFT_365`] - Trust email share activity from the specified service(s) if the email recipient is on this domain.
        You must have an email connector configured for your tenant to support this trusted action.

    Multiple options can be supplied to specify cloud-share, cloud-sync, and email-share services.

    For example, the following command will create a trusted domain that trusts file-uploads to the domain and cloud sync events from `BOX` and `ICLOUD`.

        trusted-activities add domain --file-upload --cloud-sync-services BOX --cloud-sync-services ICLOUD

    """
    client = Client()
    try:
        activity = client.trusted_activities.v2.add_domain(
            domain,
            description=description,
            file_upload=file_upload,
            git_push=git_push,
            cloud_sync_services=cloud_sync_services,
            cloud_share_services=cloud_share_services,
            email_share_services=email_share_services,
        )
    except MissingActivityActionGroupsError:
        raise click.UsageError(
            "At least one activity must be trusted for this domain.  Use 'incydr trusted-activities add domain --help' to see available options for configuration."
        )
    _output_trusted_activity(activity, format_, client.settings.use_rich)


@add.command("url-path", cls=IncydrCommand)
@click.argument("url_path")
@click.option("--description", default=None, help="Optional description.")
@single_format_option
@logging_options
def url_path_(
    url_path: str,
    description: str = None,
    format_: SingleFormat = None,
):
    """
    Trust browser uploads to only part of a domain by trusting a specific `URL_PATH` (ex: `my-domain.com/path`).
    """
    client = Client()
    activity = client.trusted_activities.v2.add_url_path(url_path, description)
    _output_trusted_activity(activity, format_, client.settings.use_rich)


@add.command(cls=IncydrCommand)
@click.argument("workspace_name")
@click.option("--description", default=None, help="Optional description.")
@single_format_option
@logging_options
def slack_workspace(
    workspace_name: str,
    description: str = None,
    format_: SingleFormat = None,
):
    """
    Trust activity uploaded through a Slack workspace specified by `WORKSPACE_NAME`.
    """
    client = Client()
    activity = client.trusted_activities.v2.add_slack_workspace(
        workspace_name, description=description
    )
    _output_trusted_activity(activity, format_, client.settings.use_rich)


@add.command(cls=IncydrCommand)
@click.argument("account_name")
@click.option("--description", default=None, help="Optional description.")
@click.option(
    "--dropbox",
    is_flag=True,
    default=False,
    help="Trust Dropbox as a cloud sync service.",
)
@click.option(
    "--one-drive",
    is_flag=True,
    default=False,
    help="Trust OneDrive as a cloud sync service.",
)
@single_format_option
@logging_options
def account(
    account_name: str,
    description: str = None,
    dropbox: bool = False,
    one_drive: bool = False,
    format_: SingleFormat = None,
):
    """
    Trust activity for a specific corporate account specified by `ACCOUNT_NAME` for cloud sync apps installed on user devices.

    Use the `--dropbox` and/or `--one-drive` options to indicate trusted cloud sync services for this account.
    """
    client = Client()
    activity = client.trusted_activities.v2.add_account_name(
        account_name, description=description, dropbox=dropbox, one_drive=one_drive
    )
    _output_trusted_activity(activity, format_, client.settings.use_rich)


@add.command(cls=IncydrCommand)
@click.argument("git_uri")
@click.option("--description", default=None, help="Optional description.")
@single_format_option
@logging_options
def git_repo(
    git_uri: str,
    description: str = None,
    format_: SingleFormat = None,
):
    """
    Trust file upload activity to a git repository.  Requires a `GIT_URI` path (ex: `bitbucket.org:exampleent/myrepo`).
    """
    client = Client()
    activity = client.trusted_activities.v2.add_git_repository(
        git_uri, description=description
    )
    _output_trusted_activity(activity, format_, client.settings.use_rich)


def _output_trusted_activity(
    model: TrustedActivity, format_: SingleFormat, use_rich: bool = True
):
    if format_ == SingleFormat.rich and use_rich:
        t = Table(title=f"Trusted Activity {model.activity_id}")
        t.add_column("Info")
        t.add_column("Action Groups")

        # exclude activity action groups from the info panel
        include = list(TrustedActivity.model_fields.keys())
        include.remove("activity_action_groups")
        t.add_row(
            model_as_card(
                model,
                include=include,
            ),
            model_as_card(model, include=["activity_action_groups"]),
        )

        console.print(t)
    elif format_ == SingleFormat.json_pretty:
        console.print_json(model.json())
    else:
        click.echo(model.json())



from requests import Response

from _incydr_sdk.orgs.models import Org
from _incydr_sdk.orgs.models import OrgsList


class OrgsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = OrgsV1(self._parent)
        return self._v1


class OrgsV1:
    """Client for `/v1/orgs` endpoints.

    Usage example:

        >>> import incydr
        >>>
        >>> client = incydr.Client(**kwargs)
        >>> client.orgs.v1.list_orgs()
    """

    def __init__(self, parent):
        self._parent = parent

    def activate(self, org_guid: str) -> Response:
        """
        Activate an org.

        **Parameters:**

        * **org_guid**: `str` (required) - The unique ID for the org.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.post(f"/v1/orgs/{org_guid}/activate")

    def list(self, active: bool = None) -> OrgsList:
        """
        List orgs.

        **Parameters:**

        * **active**: `bool` - Return only orgs matching this active state. Defaults to None, which will return both active and inactive orgs.

        **Returns**: An [`OrgsList`][orgslist-model] object.
        """
        return OrgsList.parse_response(
            self._parent.session.get("/v1/orgs", params={"active": active})
        )

    def create(
        self,
        org_name: str,
        org_ext_ref: str = None,
        parent_org_guid: str = None,
        notes: str = None,
    ) -> Org:
        """
        Create an org.

        **Parameters:**

        * **org_name**: `str` (required) - The name of the org to create.
        * **org_ext_ref**: `str` - The external reference of the org to create. Defaults to None.
        * **parent_org_guid**: `str` - The parent ID of the org to create. Defaults to None.
        * **notes**: `str` - The notes of the org to create. Defaults to None.
        pa
        **Returns**: An [`Org`][org-model] object representing the created org.
        """
        # Ensure parent org guid
        if not parent_org_guid:
            orgslist = self.list().orgs
            id_list = list(map(lambda x: x.org_guid, orgslist))
            for org in orgslist:
                if org.parent_org_guid not in id_list:
                    parent_org_guid = org.org_guid
                    break
        payload = {
            "orgName": org_name,
            "orgExtRef": org_ext_ref,
            "parentOrgGuid": parent_org_guid,
            "notes": notes,
        }
        return Org.parse_response(self._parent.session.post("/v1/orgs", json=payload))

    def deactivate(self, org_guid: str) -> Response:
        """
        Deactivate an org.

        **Parameters:**

        * **org_guid**: `str` (required) - The unique ID for the org.

        **Returns**: A `requests.Response` indicating success.
        """
        return self._parent.session.post(f"/v1/orgs/{org_guid}/deactivate")

    def get_org(self, org_guid: str) -> Org:
        """
        Get a specific organization.

        **Parameters**:

        * **org_guid**: `str` (required) - The unique ID for the org.

        **Returns**: An [`Org`][org-model] object representing the org.
        """
        return Org.parse_response(self._parent.session.get(f"/v1/orgs/{org_guid}"))

    def update(
        self, org_guid: str, org_name: str, org_ext_ref: str = None, notes: str = None
    ) -> Org:
        """
        Update an org.

        **Parameters:**

        * **org_guid**: `str` (required) - The unique ID for the org to update.
        * **org_name**: `str` (required) - The name of the org to update.
        * **org_ext_ref**: `str` - The external reference of the org to update. Defaults to None.
        * **notes**: `str` - The notes of the org to update. Defaults to None.

        **Returns**: An [`Org`][org-model] object representing the created org.
        """
        payload = {"orgName": org_name, "orgExtRef": org_ext_ref, "notes": notes}
        return Org.parse_response(
            self._parent.session.put(f"/v1/orgs/{org_guid}", json=payload)
        )



from itertools import count

from _incydr_sdk.departments.models import GetPageRequest
from _incydr_sdk.directory_groups.models import DirectoryGroupsPage


class DirectoryGroupsClient:
    def __init__(self, parent):
        self._parent = parent
        self._v1 = None

    @property
    def v1(self):
        if self._v1 is None:
            self._v1 = DirectoryGroupsV1(self._parent)
        return self._v1


class DirectoryGroupsV1:
    """
    Client for `/v1/departments` endpoints.

    Usage example:

        >>> import incydr
        >>> client = incydr.Client(**kwargs)
        >>> client.directory_groups.v1.get_page()
    """

    def __init__(self, parent):
        self._parent = parent

    def get_page(
        self, page_num: int = 1, page_size=None, name=None
    ) -> DirectoryGroupsPage:
        """
        Get a page of directory groups. Retrieves directory group information that has been pushed to Code42 from SCIM or User Directory Sync.
        The resulting group IDs can be used to include directory groups on watchlists.

        **Parameters**:

        * **page_num**: `int` - Page number for results, starting at 1.
        * **page_size**: `int` - Max number of results to return for a page.
        * **name**: `str` - Matches directory groups whose name is like the given value.

        **Returns**: A [`DirectoryGroupsPage`][directorygroupspage-model] object.
        """

        data = GetPageRequest(
            page=page_num,
            page_size=page_size or self._parent.settings.page_size,
            name=name,
        )
        response = self._parent.session.get("/v1/directory-groups", params=data.dict())
        return DirectoryGroupsPage.parse_response(response)

    def iter_all(self, page_size=None, name=None):
        """
        Iterate over all directory groups.

        Accepts the same parameters as `.get_page()` excepting `page_num`.

        **Returns**: A generator yielding individual [`DirectoryGroup`][directorygroup-model] objects.
        """
        page_size = page_size or self._parent.settings.page_size
        for page_num in count(1):
            page = self.get_page(page_num=page_num, page_size=page_size, name=name)
            yield from page.directory_groups
            if len(page.directory_groups) < page_size:
                break



import click

from _incydr_cli.cmds.options.utils import user_lookup_callback

manager_option = click.option(
    "--manager",
    default=None,
    help="Filter by manager.  Accepts a manager's user ID or username, performs an additional lookup if a username is passed.",
    callback=user_lookup_callback,
)
title_option = click.option("--title", default=None, help="Filter by job title.")
division_option = click.option("--division", default=None, help="Filter by division.")
department_option = click.option(
    "--department", default=None, help="Filter by department."
)
employment_type_option = click.option(
    "--employment-type", default=None, help="Filter by employment type."
)
country_option = click.option("--country", default=None, help="Filter by country.")
region_option = click.option("--region", default=None, help="Filter by region (state).")
locality_option = click.option(
    "--locality", default=None, help="Filter by locality (city)."
)
active_option = click.option(
    "--active/--inactive",
    default=None,
    help="Filter by active or inactive users. Defaults to returning both when when neither option is passed.",
)
deleted_option = click.option(
    "--deleted/--non-deleted",
    default=None,
    help="Filter by deleted or non-deleted users. Defaults to returning both when when neither option is passed.",
)
support_user_option = click.option(
    "--support-user/--non-support-user",
    default=None,
    help="Filter by support users or non-support users. Defaults to returning both when when neither option is passed.",
)


def profile_filter_options(f):
    f = manager_option(f)
    f = title_option(f)
    f = division_option(f)
    f = department_option(f)
    f = employment_type_option(f)
    f = country_option(f)
    f = region_option(f)
    f = locality_option(f)
    f = active_option(f)
    f = deleted_option(f)
    f = support_user_option(f)
    return f



from enum import Enum

import click

from _incydr_cli.core import incompatible_with


class TableFormat(str, Enum):
    table = "table"
    json_pretty = "json-pretty"
    json_lines = "json-lines"
    csv = "csv"


class SingleFormat(str, Enum):
    rich = "rich"
    json_pretty = "json-pretty"
    json_lines = "json-lines"


input_format_option = click.option(
    "--format",
    "-f",
    "format_",
    type=click.Choice(["csv", "json-lines"]),
    default="csv",
    help="Specify format of input file: 'csv' or 'json-lines'.  Defaults to 'csv'.",
)

table_format_option = click.option(
    "--format",
    "-f",
    "format_",
    type=TableFormat,
    help="Format to print result. One of 'table', 'json-pretty', 'json-lines', or 'csv. If environment has INCYDR_USE_RICH=false set, defaults to 'json-lines', else defaults to 'table'.",
    default=TableFormat.table,
)
single_format_option = click.option(
    "--format",
    "-f",
    "format_",
    default="rich",
    type=SingleFormat,
    help="Format to print result. One of 'rich', 'json-pretty', or 'json-lines'. If environment has INCYDR_USE_RICH=false set, defaults to 'json-lines', else defaults to 'rich'.",
)

columns_option = click.option(
    "--columns",
    default=None,
    help="Comma-delimited string of column names. Nested values should be specified in dot-notation. "
    "Limits output to contain only the specified columns in CSV or Table format.  Ignored for JSON output formats.",
    callback=lambda ctx, param, value: value.split(",") if value is not None else None,
)

output_option = click.option(
    "--output",
    default=None,
    help="Use to send the raw-json data in to a syslog server.  Pass a string in the format PROTOCOL:HOSTNAME:PORT to output "
    "to the specified server endpoint, where format is either TCP, TLS-TCP, or UDP (ex: TCP:localhost:5000). "
    "Also accepts strings of the format HOSTNAME and HOSTNAME:PORT. Defaults to TCP protocol on port 601. "
    "The --certs or --ignore-cert-validation option can be used with TLS-TCP format.  Note that most data will be too large to be sent "
    "via UDP protocol.",
    cls=incompatible_with(["format"]),
)
ignore_cert_validation_option = click.option(
    "--ignore-cert-validation",
    default=False,
    help="Set to skip CA certificate validation for the TLS-TCP protocol. Incompatible with the 'certs' option.",
    cls=incompatible_with(["certs"]),
)
certs_option = click.option(
    "--certs",
    default=None,
    help="A CA certificates-chain file for the TLS-TCP protocol.",
)


def output_options(f):
    f = output_option(f)
    f = certs_option(f)
    f = ignore_cert_validation_option(f)
    return f



from typing import Optional

import click
from rich.panel import Panel

from _incydr_cli import console
from _incydr_cli import logging_options
from _incydr_cli import render
from _incydr_cli.cmds.options.output_options import columns_option
from _incydr_cli.cmds.options.output_options import single_format_option
from _incydr_cli.cmds.options.output_options import SingleFormat
from _incydr_cli.cmds.options.output_options import table_format_option
from _incydr_cli.cmds.options.output_options import TableFormat
from _incydr_cli.core import IncydrCommand
from _incydr_cli.core import IncydrGroup
from _incydr_sdk.core.client import Client
from _incydr_sdk.legal_hold.models import Custodian
from _incydr_sdk.legal_hold.models import CustodianMatter
from _incydr_sdk.legal_hold.models import LegalHoldPolicy
from _incydr_sdk.legal_hold.models import Matter
from _incydr_sdk.utils import model_as_card


@click.group(cls=IncydrGroup)
@logging_options
def legal_hold():
    """View and manage legal holds."""


@legal_hold.command("list-matters-for-user", cls=IncydrCommand)
@click.argument("user-id")
@table_format_option
@columns_option
@logging_options
def list_matters_for_user(user_id: str, format_: TableFormat, columns: Optional[str]):
    """List the matter memberships for a specific user."""
    client = Client()
    memberships_ = client.legal_hold.v1.iter_all_memberships_for_user(user_id=user_id)

    if format_ == TableFormat.csv:
        render.csv(CustodianMatter, memberships_, columns=columns, flat=True)
    elif format_ == TableFormat.table:
        render.table(CustodianMatter, memberships_, columns=columns, flat=False)
    elif format_ == TableFormat.json_pretty:
        for item in memberships_:
            console.print_json(item.json())
    else:
        for item in memberships_:
            click.echo(item.json())


@legal_hold.command("list-custodians", cls=IncydrCommand)
@click.argument("matter-id")
@table_format_option
@columns_option
@logging_options
def list_custodians(matter_id: str, format_: TableFormat, columns: Optional[str]):
    """List the custodians for a specific matter."""
    client = Client()
    memberships_ = client.legal_hold.v1.iter_all_custodians(matter_id=matter_id)

    if format_ == TableFormat.csv:
        render.csv(Custodian, memberships_, columns=columns, flat=True)
    elif format_ == TableFormat.table:
        render.table(Custodian, memberships_, columns=columns, flat=False)
    elif format_ == TableFormat.json_pretty:
        for item in memberships_:
            console.print_json(item.json())
    else:
        for item in memberships_:
            click.echo(item.json())


@legal_hold.command("add-custodian", cls=IncydrCommand)
@click.option("--user-id", required=True, default=None, help="The user ID to add.")
@click.option(
    "--matter-id",
    required=True,
    default=None,
    help="The matter ID to which the user will be added",
)
@single_format_option
@logging_options
def add_custodian(user_id: str, matter_id: str, format_: SingleFormat):
    """Add a custodian to a matter."""
    client = Client()
    result = client.legal_hold.v1.add_custodian(user_id=user_id, matter_id=matter_id)

    if format_ == SingleFormat.rich and client.settings.use_rich:
        console.print(
            Panel.fit(
                model_as_card(result), title=f"Membership: {result.custodian.username}"
            )
        )
    elif format_ == SingleFormat.json_pretty:
        console.print_json(result.json())
    else:
        click.echo(result.json())


@legal_hold.command("remove-custodian", cls=IncydrCommand)
@click.option("--user-id", required=True, default=None, help="The user ID to remove.")
@click.option(
    "--matter-id",
    required=True,
    default=None,
    help="The matter ID from which the user will be removed",
)
@logging_options
def remove_custodian(
    user_id: str,
    matter_id: str,
):
    """Remove custodian from a matter."""
    client = Client()
    client.legal_hold.v1.remove_custodian(user_id=user_id, matter_id=matter_id)
    console.log(f"User {user_id} removed successfully from matter {matter_id}.")


@legal_hold.command("list-matters", cls=IncydrCommand)
@click.option(
    "--creator-user-id",
    default=None,
    help="Find legal hold matters that were created by the user with this unique identifier.",
)
@click.option(
    "--active/--inactive",
    default=None,
    help="Filter by active or inactive matters. Defaults to returning both when when neither option is passed.",
)
@click.option(
    "--name",
    default=None,
    help="Find legal hold matters whose 'name' either equals or partially contains this value.",
)
@table_format_option
@columns_option
@logging_options
def list_matters(
    creator_user_id: Optional[str],
    active: Optional[bool],
    name: Optional[str],
    format_: TableFormat,
    columns: Optional[str],
):
    """List all matters."""
    client = Client()
    result = client.legal_hold.v1.iter_all_matters(
        creator_user_id=creator_user_id, active=active, name=name
    )

    if format_ == TableFormat.csv:
        render.csv(Matter, result, columns=columns, flat=True)
    elif format_ == TableFormat.table:
        render.table(Matter, result, columns=columns, flat=False)
    elif format_ == TableFormat.json_pretty:
        for item in result:
            console.print_json(item.json())
    else:
        for item in result:
            click.echo(item.json())


@legal_hold.command("create-matter", cls=IncydrCommand)
@click.option(
    "--policy-id",
    required=True,
    default=None,
    help="The policy ID to be used by the created matter. Required.",
)
@click.option(
    "--name",
    required=True,
    default=None,
    help="The name of the matter to be created. Required.",
)
@click.option(
    "--description", default=None, help="The description of the matter to be created."
)
@click.option("--notes", default=None, help="The notes for the matter to be created.")
@single_format_option
@logging_options
def create_matter(
    policy_id: str,
    name: str,
    description: Optional[str],
    notes: Optional[str],
    format_: SingleFormat,
):
    """Create a matter."""
    client = Client()
    result = client.legal_hold.v1.create_matter(
        policy_id=policy_id, name=name, description=description, notes=notes
    )

    if format_ == SingleFormat.rich and client.settings.use_rich:
        console.print(Panel.fit(model_as_card(result), title=f"Matter: {result.name}"))
    elif format_ == SingleFormat.json_pretty:
        console.print_json(result.json())
    else:
        click.echo(result.json())


@legal_hold.command("deactivate-matter", cls=IncydrCommand)
@click.argument("matter-id")
@logging_options
def deactivate_matter(matter_id: str):
    """Deactivate a matter."""
    client = Client()
    client.legal_hold.v1.deactivate_matter(matter_id=matter_id)
    console.log(f"Successfully deactivated {matter_id}")


@legal_hold.command("reactivate-matter", cls=IncydrCommand)
@click.argument("matter-id")
@logging_options
def reactivate_matter(matter_id: str):
    """Reactivate a matter."""
    client = Client()
    client.legal_hold.v1.reactivate_matter(matter_id=matter_id)
    console.log(f"Successfully reactivated {matter_id}")


@legal_hold.command("show-matter", cls=IncydrCommand)
@click.argument("matter-id")
@single_format_option
@logging_options
def show_matter(matter_id: str, format_: SingleFormat):
    """Show details for a matter."""
    client = Client()
    result = client.legal_hold.v1.get_matter(matter_id=matter_id)

    if format_ == SingleFormat.rich and client.settings.use_rich:
        console.print(Panel.fit(model_as_card(result), title=f"Matter: {result.name}"))
    elif format_ == SingleFormat.json_pretty:
        console.print_json(result.json())
    else:
        click.echo(result.json())


@legal_hold.command("list-policies", cls=IncydrCommand)
@table_format_option
@columns_option
@logging_options
def list_policies(format_: TableFormat, columns: Optional[str]):
    client = Client()
    result = client.legal_hold.v1.iter_all_policies()

    if format_ == TableFormat.csv:
        render.csv(LegalHoldPolicy, result, columns=columns, flat=True)
    elif format_ == TableFormat.table:
        render.table(LegalHoldPolicy, result, columns=columns, flat=False)
    elif format_ == TableFormat.json_pretty:
        for item in result:
            console.print_json(item.json())
    else:
        for item in result:
            click.echo(item.json())


@legal_hold.command("show-policy", cls=IncydrCommand)
@click.argument("policy-id")
@single_format_option
@logging_options
def show_policy(policy_id: str, format_: SingleFormat):
    client = Client()
    result = client.legal_hold.v1.get_policy(policy_id=policy_id)

    if format_ == SingleFormat.rich and client.settings.use_rich:
        console.print(Panel.fit(model_as_card(result), title=f"Policy: {result.name}"))
    elif format_ == SingleFormat.json_pretty:
        console.print_json(result.json())
    else:
        click.echo(result.json())
